(***********************************************************************
This file was generated automatically by the Mathematica front end.
It contains Initialization cells from a Notebook file, which typically
will have the same name as this file except ending in ".nb" instead of
".m".

This file is intended to be loaded into the Mathematica kernel using
the package loading commands Get or Needs.  Doing so is equivalent to
using the Evaluate Initialization Cells menu command in the front end.

DO NOT EDIT THIS FILE.  This entire file is regenerated automatically 
each time the parent Notebook file is saved in the Mathematica front end.
Any changes you make to this file will be overwritten.
***********************************************************************)









BeginPackage["OptimizationToolbox`LPSimplex`",
"OptimizationToolbox`CommonFunctions`"];



GaussJordanForm::usage="GaussJordanForm[mat] - converts the given matrix into a Gauss-Jordan form, printing results at all intermediate steps. The function performs row exchanges, if necessary.";



LUDecompositionSteps::usage="LUDecompositionSteps[A] - decomposes matrix A into LU form, printing all intermediate steps. Equations are reordered, if necessary.";

PerformLUDecomposition::usage="PerformLUDecomposition[A]. Built-in Mathematica functions LUDecomposition and LUBackSubstitution perform LU decomposition and solution using forward/back substitution. Equation order is changed, if necessary. The results of LU decomposition are stored in one matrix. The upper part (including diagonal) is the U matrix and the lower part (with 1's understood on the diagonal) is the L matrix. The PerformLUDecomposition function takes the results of LUDecomposition and explicitly writes out L and U matrices.";



BasicSolutions::usage="BasicSolutions[f, eqns, vars, maxSolutions:20] - determines basic solutions for an LP problem written in standard form. f = the objective function, eqns = constraint equations, vars = list of variables. maxSolutions = maximum number of solutions computed (default \[LessEqual] 20).";



BasicSimplex::usage = "BasicSimplex[f, g, vars, options]. Solves an LP problem using Phase I and II Simplex algorithm. f is the objective function, g is a list of constraints, and vars is a list of variables. See Options[BasicSimplex] to find out about a list of valid options for this function.";

Options[BasicSimplex] =  
	{UnrestrictedVariables->{}, MaxIterations->10,
	ProblemType->Min, SimplexVariables->{"x","s","a"},
	PrintLevel->1,
	SensitivityAnalysis->False};

SimplexVariables::usage="SimplexVariables is an option of the BasicSimplex. t specifies symbols to use when creating variable names. Default is {x, s, a}, where x is used for problem variables, s for slack/surplus, and a for artificial.";

SensitivityAnalysis::usage="SensitivityAnalysis is an option of the Simplex method. It controls whether a post-optimality (sensitivity) analysis is performed after obtaining an optimum solution. The default is False.";



RevisedSimplex::usage = "RevisedSimplex[f, g, vars, options]. Solves an LP problem using Phase I and II Simplex algorithm. f is the objective function, g is a list of constraints, and vars is a list of variables. See Options[RevisedSimplex] to find out about a list of valid options for this function.";

Options[RevisedSimplex] =  
	{UnrestrictedVariables->{}, MaxIterations->10,
	ProblemType->Min, StandardVariableName->"x",
	PrintLevel->1,
	SensitivityAnalysis->False};



Begin["`Private`"];

Needs["Utilities`FilterOptions`"];
Needs["DiscreteMath`Combinatorica`"];
Needs["LinearAlgebra`MatrixManipulation`"];





GaussJordanForm[mat_]:=BackSubs[ZeroLowerHalf[mat]]

ZeroLowerHalf[mat_]:=Module[{rows,cols,i,j,a=mat},
	Print["**********Forward pass**********"];
	{rows,cols}=Dimensions[a];
	Do[
		{s,a}=GetPivot[a,j];
		If[s!=0,
			If[s!=1,
				a[[j]]=a[[j]]/s;
				Print["(Row "<>ToString[j]<>")/"<>
					ToString[s,StandardForm]->MatrixForm[a]]
			];
			Do[s=a[[i,j]];
				If[s!=0,
					a[[i]]=a[[i]]-s*a[[j]];
					Print["(Row "<>ToString[i]<>")-("<>
					ToString[s,StandardForm]<>")*Row "<>
					ToString[j]->MatrixForm[a]]
				],
				{i,j+1,rows}
			]
		],
		{j,1,Min[rows,cols]}
	];a
];

GetPivot[a_,j_]:=Module[{m,n,ar=a,c,s=0,i=j},
	{m,n}=Dimensions[a];
	While[And[s==0,i\[LessEqual]m],
		s=ar[[j,j]];
		++i;
		If[And[s==0,i\[LessEqual]m],
			c=ar[[j]];ar[[j]]=ar[[i]];ar[[i]]=c;
			Print["Rows "<>ToString[j]<>" and "<>
			ToString[i]<>" interchanged"->MatrixForm[ar]];
			
		];
	];{s,ar}
];

BackSubs[mat_]:=Module[{rows,cols,i,j,a=mat},
	Print["**********Backward pass**********"];
	{rows,cols}=Dimensions[a];
	Do[
		If[a[[j,j]]!=0,
			Do[s=a[[i,j]];
				If[s!=0,
					a[[i]]=a[[i]]-s*a[[j]];
					Print["(Row "<>ToString[i]<>")-("<>
					ToString[s,StandardForm]<>")*Row "<>
					ToString[j]->MatrixForm[a]]
				],
				{i,1,j-1}
			]
		],
		{j,Min[rows,cols],1,-1}
	];a
];



LUDecompositionSteps[A_]:=
Module[{mi,M={},U,MA=A,L,m=Length[A],p,pmat,P={},MM},
	MM = IdentityMatrix[Length[A]];
	p=Range[m];
	(*First pass constructs U with pivoting and forms
	permutation matrices P that give the correct order of equations*)
	Do[{mi,MA,p,pmat}=makeMiWithPivoting[MA,i,p];MA=mi.MA;
		If[pmat!={},AppendTo[P,pmat]];
		If[P==={},PrintLabelledList[{mi,MA},
			{Superscript["M",i],Superscript["A",i+1]}]
		];
		MM = mi.MM;
		AppendTo[M,mi],{i,1,m-1}
	];
	(*If pivoting is necessary go back on the reordered
	 equations and form L, U*)
	If[P!={},
		Print["***Re-ordered equations***"];
		MA = Apply[Dot,P].A; M ={};
		MM = IdentityMatrix[Length[A]];
		PrintLabelledList[{MA,p},{"A","Equation order"}];
		Do[mi=makeMiWithPivoting[MA,i,p][[1]];MA=mi.MA;
		PrintLabelledList[{mi,MA},
			{Superscript["M",i],Superscript["A",i+1]}];
		MM = mi.MM;
		AppendTo[M,mi],{i,1,m-1}](*,
		MA=A;
		Do[MA=M[[i]].MA;
			PrintLabelledList[{M[[i]],MA},
			{Superscript["M",i],Superscript["A",i+1]}],
			{i,1,Length[M]}
		]*)
	];
	Print["Product of all M matrices"->MatrixForm[MM]];
	U=MA;
	L=Apply[Plus,-M];
	Do[L[[i,i]]=1,{i,1,m}];
	PrintLabelledList[{L,U,p},{"L","U","Equation order"}];
	{L,U,p}
];
makeMiWithPivoting[A_,i_,order_]:=
Module[{a=A, s=0,k=i-1,m=Length[A],M,j,p=order,pmat={}},
	M=DiagonalMatrix[Table[1,{m}]];
	While[And[s==0,++k\[LessEqual]m],
		s=a[[k,i]];
	];
	If[k!=i,
		p[[i]]=k;
		p[[k]]=i;
		c=a[[k]];
		a[[k]]=a[[i]];
		a[[i]]=c;
		pmat=makePermutationMatrix[m,k,i]
	];
	Do[M[[j,i]]=-a[[j,i]]/a[[i,i]],{j,i+1,m}];
	{M,a,p,pmat}
];
makePermutationMatrix[n_,k_,l_]:=Module[{a,b},
	a=IdentityMatrix[n];
	b=a[[k]];
	a[[k]]=a[[l]];
	a[[l]]=b;
	a
];

PerformLUDecomposition[A_]:=
Module[{m=Length[A],LU,L,U,eqnOrder,cond},
	{LU,eqnOrder,cond}=LUDecomposition[A];
	U=UpperDiagonalMatrix[LU[[#1,#2]]&,m];
	L=LowerDiagonalMatrix[LU[[#1,#2]]&,m]-
		DiagonalMatrix[Map[-1+LU[[#,#]]&,Range[m]]];
	PrintLabelledList[{L,U,eqnOrder},{"L","U","Equation Order"}];
	{LU,eqnOrder,cond}
];



BasicSolutions[f_,eqns_,vars_,maxSolutions_:20]:=
Module[{basicSets,sol},
	basicSets=KSubsets[vars,Length[eqns]];
	basicSets = Take[basicSets,Min[maxSolutions,Length[basicSets]]];
	sol=Map[BasicSolution[f,eqns,#,vars]&,basicSets];
	Print[MatrixForm[Join[{{"Basis","Solution","Status","f"}},sol]]];
	Transpose[sol][[2]]
]		

BasicSolution[f_,eqns_,basis_,vars_]:=
Module[{nonbasic,nonbasicSol,basicSol,sol,status,fval,pt},
	nonbasic=Complement[vars,basis];
	nonbasicSol=ToRules[nonbasic==Table[0,{Length[nonbasic]}]];
	basicSol=Solve[eqns/.nonbasicSol,basis];
	If[basicSol!={},
		sol=Join[basicSol[[1]],nonbasicSol];
		pt=vars/.sol;
		If[NonNegative[Min[pt]],status="Feasible";
		fval=f/.sol,status="Infeasible";
		fval="--"],
		status="NoSolution";
		fval="--";pt={"--"}
	];
	{basis,pt,status,fval}
]



ConvertUnrestricted[fcn_, vars_, unrestrictedVars_, x_] := 
Module[{xv={},p=1,i},
	Do[If[MemberQ[unrestrictedVars,i],
			AppendTo[xv,{vars[[i]]->x[[p]]-x[[p+1]]}];
			p=p+2,
			AppendTo[xv,{vars[[i]]->x[[p]]}];
			p=p+1
		],{i,1,Length[vars]}
	];		
	{Map[Expand,fcn /. Flatten[xv]],xv}
]

getConstant[poly_, vars_] := Module[{c},
	c=Map[Fold[Coefficient[#1, #2, 0]&,#,vars]&, poly];
	If[c===poly,0,c]
]; 

getCoefficients[poly_, vars_] := Map[Coefficient[poly,#]&, vars]; 

StandardizeConstraint[g_, vars_] :=
Module[{a,b, ng},
	Switch[Head[g],
		LessEqual, poly = g[[1]] - g[[2]];
			a = getCoefficients[poly,vars];
			b = getConstant[poly, vars];
			If[b<=0, ng = a.vars <= -b, ng = -a.vars >= b],
		GreaterEqual, poly = g[[1]] - g[[2]];
			a = getCoefficients[poly,vars];
			b = getConstant[poly, vars];
			If[b<=0, ng = a.vars >= -b, ng = -a.vars <= b],
		Equal, poly = g[[1]] - g[[2]];
			a = getCoefficients[poly,vars];
			b = getConstant[poly, vars];
			If[b<=0, ng = a.vars == -b, ng = -a.vars == b]
	];
	ng
]

getCoefficients[poly_, vars_] := Map[Coefficient[poly,#]&, vars]; 



BasicSimplex[f_, g_, vars_,opts___Rule]:=
Module[{freeIndices,unrestrictedVars, maxIter, type, varNames,
	prLevel, prSetup, printTableaus, sensitivity},
	{unrestrictedVars, maxIter, type, varNames, 
		prLevel, sensitivity} = 
		{UnrestrictedVariables, MaxIterations, ProblemType,
		SimplexVariables, PrintLevel, 
		SensitivityAnalysis} /. Flatten[{opts, Options[BasicSimplex]}];
	Switch[prLevel,
		0, {prSetup, printTableaus}={False, False}, 
		1, {prSetup, printTableaus}={True, False}, 
		_, {prSetup, printTableaus}={True, True}
	]; 
	freeIndices = Sort[Flatten[Map[Position[vars,#]&,unrestrictedVars]]];
	SolveLP[f, g, vars, freeIndices,
		varNames, maxIter, type, {prSetup,printTableaus}, sensitivity]
];

SolveLP[f_, g_, prbVars_, unrestrictedVars_, {x_, s_, a_},
maxIter_, type_, {prSetup_,printTableaus_}, sensitivity_] :=
Module[{designVars, sf, sg,status = "NonOptimum", sensInfo={},
	sxColumns, artVarColumns, sxVarsNum, iter = 0, vars,
	basis, tableau, rhs, cleanBasis, cleanRHS, varsMapping, i, k},
	designVars = Table[mySubscript[x,i], 
		{i,1,Length[prbVars]+Length[unrestrictedVars]}];
	varsMapping = {};
	k = 1;
	Do[If[FreeQ[unrestrictedVars,i],
		varsMapping={varsMapping,prbVars[[i]]->designVars[[k]]};
		k = k + 1,
		varsMapping={varsMapping,
			prbVars[[i]]->designVars[[k]]-designVars[[k+1]]};
		k = k + 2],
		{i,1,Length[prbVars]}
	];
	varsMapping=Flatten[varsMapping];
	{sf, sg} = ConvertToStandardLP[type, f, g, prbVars,
      	unrestrictedVars, designVars];
	{vars, sxColumns, artVarColumns, basis, tableau, rhs} =
		InitialSimplexTableau [sf, sg, {designVars, s, a}];
	If [prSetup,
	    Print["Problem variables redefined as:",varsMapping];
		Print["Minimize ", sf];
		Print["Subject to ", MatrixForm[sg]];
		Print["All variables \[GreaterEqual] 0"];
		Print["********** Initial simplex tableau **********"];
		Print["New problem variables:",vars];
		ShowSimplexTableau[vars, basis, tableau, rhs]
	];
	sxVarsNum = Length[sxColumns]; 

      (*Phase I *)
	If [artVarColumns != {},
		While[status === "NonOptimum" && iter < maxIter,
			{status, basis, tableau, rhs} = SimplexStep[sxVarsNum,
			basis, tableau, rhs, printTableaus];
			++iter;
			If[And[(status==="NonOptimum"),printTableaus],
				Print["**********Phase I - Iteration "<>
					ToString[iter]<>"**********"];
				ShowSimplexTableau[vars, basis, tableau, rhs]
			]
		];
        (*If[ Or[status ==="NoPivot",
        Intersection[basis, artVarColumns]!={}],*)
        If[Last[Flatten[rhs]]==="\[Phi]",
            If[printTableaus, Print["End of Phase I"]];
            rhs = Drop[rhs,-1]; tableau = Drop[tableau,-1],                 
            Print["No feasible solution"];
            Return[{status, basis, tableau, rhs}]
		];
	];
      
	(* Phase II *)
	status = "NonOptimum";
	iter = 0;
	While[status === "NonOptimum" && iter < maxIter,
		{status, basis, tableau, rhs} = SimplexStep[sxVarsNum,
			basis, tableau, rhs, printTableaus];
		If [And[(status === "NonOptimum"),printTableaus],
			Print["**********Phase II - Iteration "<>
			ToString[++iter]<>"**********"];
			ShowSimplexTableau[vars, basis, tableau, rhs]
		]
	];
	If[status === "Optimum",
			nonBasicColumns = Complement[sxColumns, basis];
        cleanBasis={};
        cleanRHS = {};
        Do[If[FreeQ[artVarColumns,basis[[i]]],
            cleanBasis={cleanBasis,basis[[i]]};
            cleanRHS={cleanRHS,rhs[[i]]}],
            {i,1,Length[basis]}
        ];
        cleanBasis = Flatten[cleanBasis];
        cleanRHS = Flatten[cleanRHS];
		soln = Table[0, {sxVarsNum}];
		Do[soln[[cleanBasis[[i]]]] = cleanRHS[[i]], 
			{i, 1, Length[cleanBasis]}
		];
		soln = varsMapping /. ToRules[vars[[sxColumns]]==soln];
		If [And[prSetup, Not[printTableaus]],
			Print["********** Final simplex tableau **********"];
         	ShowSimplexTableau[vars, basis, tableau, rhs]
        ];  
		If [prSetup,
			PrintLabelledList[{{{soln}},f /.soln},
			{"Optimum solution", "Optimum objective function value"},1];
			If [Min[N[Last[tableau][[nonBasicColumns]]]]==0.,
			Print["** The problem may have multiple solutions."]
			]
		];
		If[sensitivity, 
			sensInfo = PostOptimalityAnalysis[designVars,
				Drop[sxColumns,Length[designVars]], artVarColumns,
				sf, sg, basis, tableau, rhs, prSetup]
		] 
	]; 
	If[status === "NoPivot", Print["Unbounded or infeasible problem"]]; 
	{f /.soln, soln, sensInfo}
];

InitialSimplexTableau[f_, g_, {x_, s_, a_}] :=
Module[{tableau, rhs, vars = x, zeroColumn, basis,
	newColumn, sxColumns, artVarColumns, artVarRows, wRow},
	
	tableau = Map[getCoefficients[g[[#,1]],vars]&,Range[Length[g]]];
	rhs = Append[Map[g[[#,2]]&,Range[Length[g]]],"f"];
	AppendTo[tableau, getCoefficients[f,vars]];
	zeroColumn = Table[{0},{Length[g]+1}];
	basis = Table[0,{Length[g]}];
	(* Add slack/surplus variables *)
	Do[
		newColumn = zeroColumn;
		Switch[Head[g[[i]]],
            LessEqual, AppendTo[vars, mySubscript[s,i]];
            basis[[i]] = Length[vars];
            newColumn[[i]] = {1};
            tableau = AppendRows[tableau,newColumn],
            GreaterEqual, AppendTo[vars, mySubscript[s,i]];
            newColumn[[i]] = {-1};
            tableau = AppendRows[tableau,newColumn]
         ],
         {i,1,Length[g]}
      ];
      sxColumns = Range[Length[vars]];
      (* Add artificial variables *)
      Do[
		newColumn = zeroColumn;
		Switch[Head[g[[i]]],
			GreaterEqual, AppendTo[vars, mySubscript[a,i]];
			basis[[i]] = Length[vars];
			newColumn[[i]] = {1};
			tableau = AppendRows[tableau,newColumn],
            Equal, AppendTo[vars, mySubscript[a,i]];
            basis[[i]] = Length[vars];
            newColumn[[i]] = {1};
            tableau = AppendRows[tableau,newColumn]
         ],
         {i,1,Length[g]}
      ];
      artVarColumns = Range[Length[sxColumns]+1,Length[vars]];
      artVarRows = Flatten[Map[Position[basis,#]&, artVarColumns]];
      If [artVarColumns != {},
          wRow = Join[Table[0, {Length[sxColumns]}],
          Table[1, {Length[artVarColumns]}]] -
                 Apply[Plus, Map[tableau[[#]]&,artVarRows]];
          AppendTo[tableau, wRow];
          AppendTo[rhs, "\[Phi]" - Apply[Plus, Map[rhs[[#]]&,artVarRows]]]
      ];  
           
      {vars, sxColumns, artVarColumns, basis, tableau, rhs}
	]

ConvertToStandardLP[type_, f_, g_, vars_, unrestrictedVars_, x_] :=
Module[{nf = f, ng = g, nh = h,xv},
	If[type===Max, nf = -f];
	ng = Map[StandardizeConstraint[#,vars]&, g]; 
	{nf,xv} = ConvertUnrestricted[nf, vars, unrestrictedVars, x];
	ng = Map[ConvertUnrestricted[#, vars, unrestrictedVars, x]&,ng];
	ng = First[Transpose[ng]];
	{nf, ng}
]       

(*ShowSimplexTableau[vars, basis, a, b], prints the Simplex tableau.
a and b contain coefficient matrix and the rhs respectively.*)
ShowSimplexTableau[vars_, basis_, a_, b_] :=
Module[{rows, cols, rowHeads, colHeads, line, tableau},
	rows = Length[b];
	cols = Length[vars];
	rowHeads = Join[Map[{#}&,basis], {{"Obj."}, {"ArtObj."}}];
	rowHeads = rowHeads[[Range[rows]]];
	tableau = AppendRows[rowHeads, a];
	tableau = AppendRows[tableau, Map[{#}&,b]];
	colHeads = Join[{"Basis"}, Range[cols],{"rhs"}]; 
	line = Table["-",{cols + 2}];
	tableau = Join[{colHeads, line}, tableau];
	Print[MatrixForm[tableau]]
]

SimplexStep[sxVarsNum_, basis_, tableau_, rhs_, printTableaus_] :=
Module[{cRow, pivotColumn, pivotRow, goodRatios={}, allRatios,
	newBasis = basis, newTableau = tableau, newRhs = rhs, a, b,
	pa, pb, i,r, minRatio=$MaxNumber},
	cRow = Last[tableau][[Range[sxVarsNum]]];
	If [Apply[And,Map[NonNegative,cRow]], 
		Return[{"Optimum", basis, tableau, rhs}]
	];
	pivotColumn = Position[cRow, Min[cRow]][[1,1]];
	a = Flatten[TakeColumns[tableau,{pivotColumn}]][[Range[Length[basis]]]];
	If [Max[a]\[LessEqual]0, Return[{"NoPivot", basis, tableau, rhs}]];
	b = rhs[[Range[Length[basis]]]];
	goodRatios = Table[If[a[[i]] > 0, b[[i]]/a[[i]],\[Infinity]],{i,1,Length[a]}];
	minRatio = Min[goodRatios];
	pivotRow = Last[Flatten[Position[goodRatios,minRatio,1]]];
	If[printTableaus,
		PrintLabelledList[{pivotColumn,{goodRatios},basis[[pivotRow]]},
		{"Variable to be made basic",
		"Ratios: rhs/Column "<>ToString[pivotColumn],
		"Variable out of the basic set"},1]
	];
	pa = newTableau[[pivotRow]]/tableau[[pivotRow,pivotColumn]];
	pb = newRhs[[pivotRow]]/tableau[[pivotRow,pivotColumn]];
	newBasis[[pivotRow]] = pivotColumn;
	newTableau[[pivotRow]] = pa; newRhs[[pivotRow]] = pb;
	Do[If[i != pivotRow, 
		{newTableau[[i]], newRhs[[i]]}=
		RowOperation[pivotColumn, pa, tableau[[i]], pb, rhs[[i]]]],
		{i,1,Length[tableau]}
	];
	{"NonOptimum", newBasis, newTableau, newRhs} 
] 

RowOperation[pivCol_, pa_, a_, pb_, b_] := 
Module[{ma, mb},
	ma = a - pa*a[[pivCol]];
	mb = b - pb*a[[pivCol]];
	{ma, mb}
]

PostOptimalityAnalysis[x_,sColumns_, aColumns_,f_, g_, basis_, a_, b_, prResults_] :=
Module[{cRow, u, ns = 0, na = 0, rhsChanges = {}, costChanges = {},
	c, cx={}, ax={}, nonBasicColumns={}},
	(*Recover Lagrange multipliers*)
	cRow = Last[a]; u = Table[0, {Length[g]}];
	Do[
		If[Head[g[[i]]]===LessEqual,
			u[[i]] = cRow[[sColumns[[++ns]]]],
			u[[i]] = cRow[[aColumns[[++na]]]]];
				 If[Head[g[[i]]]===GreaterEqual,++ns],
         {i,1,Length[g]}
      ];
      If[prResults, Print["Lagrange multipliers"->u]];
      
      (* RHS ranges *)
      ns = 0; na = 0;
      Do[
         If[Head[g[[i]]]===LessEqual,
            AppendTo[rhsChanges, rhsRange[Flatten[TakeColumns[a,
            {sColumns[[++ns]]}]], b]],
            AppendTo[rhsChanges, rhsRange[Flatten[TakeColumns[a,
            {aColumns[[++na]]}]], b]];
			If[Head[g[[i]]]===GreaterEqual,++ns]
		],
		{i,1,Length[g]}
	];
	If[prResults,
		Print["Allowable constraint rhs changes and ranges"];
		Print[TableForm[Table[{
		rhsChanges[[i,1]]\[LessEqual]mySubscript["\[CapitalDelta]b",i]\[LessEqual]rhsChanges[[i,2]],
		g[[i,2]]+rhsChanges[[i,1]]\[LessEqual]mySubscript["b",i]\[LessEqual]
			g[[i,2]]+rhsChanges[[i,2]]},
			{i,1,Length[rhsChanges]}]]]
	];

	(* Cost coefficient ranges *)
	ns = 0; na = 0;
	c = getCoefficients[f, x];
	(*Extract just the columns associated with nonbasic variables*)
	na=Max[If[sColumns=={},0,Last[sColumns]],Length[x]];
	nonBasicColumns = Complement[Range[na], basis];
	If[nonBasicColumns!={},
		ax = Transpose[
			Map[Flatten[TakeColumns[a, {#}]]&, nonBasicColumns]];
		cx = Last[ax]
	];
	Do[
		rowNum = Flatten[Position[basis, i]];
		If[rowNum == {},
            AppendTo[costChanges, {-cRow[[i]], \[Infinity]}],
            If[nonBasicColumns=={},
            	AppendTo[costChanges, {0,0}],
            	AppendTo[costChanges, 
            		costRange[ax[[rowNum[[1]]]], cx]]
            ]
		],
		{i,1,Length[x]}
	];
	If[prResults,
		Print["Allowable cost coefficient changes and ranges"];
		Print[TableForm[Table[{
		costChanges[[i,1]]\[LessEqual]mySubscript["\[CapitalDelta]c",i]\[LessEqual]costChanges[[i,2]],
		c[[i]]+costChanges[[i,1]]\[LessEqual]mySubscript["c",i]\[LessEqual]
			c[[i]]+costChanges[[i,2]]},
			{i,1,Length[costChanges]}]]]
	];
	{u, rhsChanges, costChanges}
]      

rhsRange[a_, b_] :=
Module[{lower = -\[Infinity], upper = \[Infinity]},
	Do[
		Which[a[[i]] > 0, lower = Max[lower, -b[[i]]/a[[i]]],
			a[[i]] < 0, upper = Min[upper, -b[[i]]/a[[i]]]
		],
		{i, 1, Length[a]-1}
	];
	{lower, upper}
]; 

costRange[a_, c_] :=
Module[{lower = -\[Infinity], upper = \[Infinity]},
	Do[
		Which[a[[i]] < 0, lower = Max[lower, c[[i]]/a[[i]]],
			a[[i]] > 0, upper = Min[upper, c[[i]]/a[[i]]]
		],
		{i, 1, Length[a]}
	];
	{lower, upper}
]; 



RevisedSimplex[f_, g_, designVars_,opts___Rule]:=
Module[{n,freeIndices,unrestrictedVars, maxIter, type, varName,
	prLevel, prSetup, prResults, sensitivity, vars, sf, sg,sol,basic, w,
	sVars, artVars, artNum, lagMult, \[CapitalDelta]c, rhsRange, costRange, \[CapitalDelta]b, 
	sensInfo={}, prSensInfo, cleanBasis,
	c, A, b, iter=0, status="NonOptimum",x,phase,intResults,xv},
	{unrestrictedVars, maxIter, type, varName, 
		prLevel, sensitivity} = 
		{UnrestrictedVariables, MaxIterations, ProblemType,
		StandardVariableName, PrintLevel, 
		SensitivityAnalysis} /. Flatten[{opts,Options[RevisedSimplex]}];
	Switch[prLevel,
		0, {prSetup, prResults, prSensInfo}={False, False, False}, 
		1, {prSetup, prResults, prSensInfo}={True, False, False}, 
		_, {prSetup, prResults, prSensInfo}={True, True, True}
	]; 
	freeIndices = Sort[Flatten[Map[
		Position[designVars,#]&,unrestrictedVars]]];
	{sf, sg,vars,basic,sVars,artVars,xv} =
		ToStandardSimplexForm[type, f, g,
		designVars,freeIndices, varName, prSetup];
	n=Length[vars];
	c=Coefficient[sf,vars];
	b=Map[Last[#]&,sg];
	A=Map[Coefficient[First[#],vars]&,sg];
	If[prSetup,
		PrintLabelledList[{{vars},{c}},{"Variables","c"},1];
		PrintLabelledList[{b,A},{"b","A"}]
	];
	x=Table[0,{n}];
	x[[basic]]=LinearSolve[Transpose[Transpose[A][[basic]]],b];

	(* Phase I *)
	If[artVars!={},
		artNum = Length[artVars];
		phase = 1;
		w=Table[0,{n}];
		w[[artVars]]=Table[1,{artNum}];
		If[prSetup,
			Print["Artificial Objective Function"->w.vars]
		];
		While[And[status==="NonOptimum",iter\[LessEqual]maxIter],
			++iter;
			If[prResults,
				StylePrint["***** Iteration "<>ToString[iter]<>
				" (Phase "<>ToString[phase]<>") *****",
				"Output",FontWeight->"Bold"];
				PrintLabelledList[{{vars[[basic]]},{x[[basic]]},
					{vars[[Complement[Range[Length[x]],basic]]]}},
					{"Basic variables","Values of basic variables",
					"Nonbasic variables"},1];
				PrintLabelledList[{c.x,w.x},{"f","Art. Obj."}]
			];
			{x,basic,status}=RevisedSimplexStep[vars,A, w, x, basic,
				prResults]
		];
		vars = Drop[vars,-artNum];
		cleanBasis = {};
		Do[If[basic[[i]]\[LessEqual]Length[vars],
			cleanBasis={cleanBasis,basic[[i]]}],
			{i,1,Length[basic]}
		];
		basic=Flatten[cleanBasis];
		x = Drop[x,-artNum];
		A = Transpose[Drop[Transpose[A],-artNum]];
		c = Drop[c,-artNum];
	];
	(* Phase II *)
	status = "NonOptimum";
	While[And[status==="NonOptimum",iter<maxIter],
		++iter;
		phase = 2;
		If[prResults,
			StylePrint["***** Iteration "<>ToString[iter]<>
			" (Phase "<>ToString[phase]<>") *****","Output",
			FontWeight->"Bold"];
			PrintLabelledList[{{vars[[basic]]},{x[[basic]]},
				{vars[[Complement[Range[Length[x]],basic]]]}},
				{"Basic variables","Values of basic variables",
					"Nonbasic variables"},1];
			PrintLabelledList[{c.x},{"f"}]
		];
		{x,basic,status}=RevisedSimplexStep[vars,A,c,x,basic,prResults]
	];
	sol = xv/.ToRules[vars==x];
	If[prResults,
		StylePrint["***** "<>ToString[status]<>" solution after "<>
		ToString[iter]<>" iterations *****","Output",FontWeight->"Bold"];
		PrintLabelledList[{{{vars[[basic]]}},{{sol}},c.x},
		{"Basis","Variable values","Objective function"},1]
	];
	If[And[sensitivity,status==="Optimum"],
		If[prSetup,
			StylePrint["Lagrange multipliers","Output",FontWeight->"Bold"]
		];
		lagMult = LagrangeMultipliersRevisedSimplex[vars, basic, 
			A, c, prSetup];
		If[prSetup,
			StylePrint["Sensitivity to constraint constants",
			"Output",FontWeight->"Bold"]
		];
		\[CapitalDelta]b=IdentityMatrix[Length[b]];
		rhsRange = Map[rhsRangeRevisedSimplex[vars, basic, A,
			b, #, prSensInfo]&,\[CapitalDelta]b];
		If[prSetup,
			Print["Allowable constraint rhs changes and ranges"];
			Print[TableForm[Table[{rhsRange[[i,1]]\[LessEqual]
			mySubscript["\[CapitalDelta]b",i]\[LessEqual]rhsRange[[i,2]],
			b[[i]]+rhsRange[[i,1]]\[LessEqual]mySubscript["b",i]\[LessEqual]b[[i]]+
			rhsRange[[i,2]]},
			{i,1,Length[\[CapitalDelta]b]}]]]
		];
		If[prSetup,
			StylePrint["Sensitivity to objective function coefficients",
			"Output",FontWeight->"Bold"]
		];
		\[CapitalDelta]c=Drop[IdentityMatrix[Length[c]],-Length[sVars]];
		costRange = Map[CostRangeRevisedSimplex[vars, basic, A, 
			c, #, prSensInfo]&, \[CapitalDelta]c];
		If[prSetup,
			Print["Allowable cost coefficient changes and ranges"];
			Print[TableForm[Table[{costRange[[i,1]]\[LessEqual]
				mySubscript["\[CapitalDelta]c",i]\[LessEqual]costRange[[i,2]],
			c[[i]]+costRange[[i,1]]\[LessEqual]mySubscript["c",i]\[LessEqual]
			c[[i]]+costRange[[i,2]]},
			{i,1,Length[\[CapitalDelta]c]}]]]
		];
		sensInfo = {lagMult,rhsRange,costRange};  
	];
	{f/.sol, sol, sensInfo}
];

RevisedSimplexStep[vars_,A_,c_,x_,basic_,PrintResults_:False]:=
Module[{B,cB,w,r,status="NonOptimum",pp,p,qq,q,d,ratios,\[Alpha],
	newx=x,newBasic=basic,nonBasic,intResults,y=x[[basic]]},
	nonBasic=Complement[Range[Length[x]],basic];
	B=Transpose[Transpose[A][[basic]]];
	cB=c[[basic]];
	w=LinearSolve[Transpose[B],cB];
	r=Map[c[[#]]-w.Transpose[A][[#]]&,nonBasic];
	If[PrintResults,
		PrintLabelledList[{B,cB,w},
		{"B",mySubscript["c","B"],"w"},3];
		PrintLabelledList[{{r}},{"Reduced cost coefficients, r"}];
	];
	If[Min[r]\[GreaterEqual]0,status="Optimum";Return[{x,basic,status}]];
	pp=First[Position[Map[Negative,r],True]];
	p=nonBasic[[pp]];
	d=-LinearSolve[B,Transpose[A][[First[p]]]];
	If[Select[d,Negative]==={},status="Unbounded";
		Return[{x,basic,status}]];
	ratios=Table[If[d[[i]]<0,-y[[i]]/d[[i]],\[Infinity]],{i,1,Length[d]}];
	\[Alpha]= Min[ratios];
	qq=First[Position[ratios,\[Alpha],{1}]];
	q=basic[[qq]];
	newx[[p]]=\[Alpha];
	newx[[basic]]=x[[basic]]+\[Alpha] d;
	newBasic = Complement[Union[basic,p],q];
	If[PrintResults,
		PrintLabelledList[{x[[basic]],d,ratios},
		{mySubscript["x","B"],"d","Ratios, -x/d"},3];
		PrintLabelledList[{vars[[p]],vars[[q]],\[Alpha],{newx}},
		{"New basic variable","Variable going out of basic set",
		"Step length, \[Alpha]","New variable values"},1]
	];
	{newx,newBasic,status}
];

ToStandardSimplexForm[type_, f_, g_, designVars_, 
unrestrictedVars_, varName_, prSetup_] :=
Module[{n, nf = f, ng = g, h={}, con, vars, artVars={},
	sVars={}, basic={},xv},
	n=Length[designVars]+Length[unrestrictedVars];
	vars = Table[mySubscript[varName,i], {i,1,n}];
	If[type===Max, nf = -f];
	{nf,xv} = ConvertUnrestricted[nf, designVars, unrestrictedVars, vars];
	ng = Map[ConvertUnrestricted[#, designVars,
		unrestrictedVars, vars]&,ng];
	{ng,xv}=Transpose[ng];
	xv=Flatten[Union[xv]];
	ng = Map[StandardizeConstraint[#, vars]&,ng];
    (* Add slack/surplus variables *)
	Do[	con=ng[[i]];
		Switch[Head[con],
			LessEqual, newVar=mySubscript[varName,Length[vars]+1];
				ng[[i]]=First[con] + newVar<=Last[con];
				AppendTo[vars,newVar];
				AppendTo[sVars,Length[vars]];
				AppendTo[basic,Length[vars]],
			GreaterEqual, newVar=mySubscript[varName,Length[vars]+1];
				ng[[i]]=First[con] - newVar>=Last[con];
				AppendTo[vars,newVar];
				AppendTo[sVars,Length[vars]]
		],{i,1,Length[ng]}
	];
    (* Add artificial variables *)
	Do[	con=ng[[i]];
		Switch[Head[con],
			LessEqual,
				AppendTo[h,First[con]==Last[con]],
			GreaterEqual, newVar=mySubscript[varName,Length[vars]+1];
				AppendTo[h,First[con] + newVar==Last[con]];
				AppendTo[vars,newVar];
				AppendTo[artVars,Length[vars]],
			Equal, newVar=mySubscript[varName,Length[vars]+1];
				AppendTo[h,First[con] + newVar==Last[con]];
				AppendTo[vars,newVar];
				AppendTo[artVars,Length[vars]]
		],{i,1,Length[ng]}
	];
	If[prSetup,
		Print["**** Problem in Starting Simplex Form ****"];
		Print["New variables are defined as "->xv];	
		If[sVars!={},Print["Slack/surplus variables"->vars[[sVars]]]];
		If[artVars!={},Print["Artificial variables"->vars[[artVars]]]];
		Print["Minimize ", nf];
		Print["Subject to ", MatrixForm[h]];
		Print["All variables \[GreaterEqual] 0"]
	];
	{nf, h, vars, Union[basic,artVars], sVars, artVars,xv}
]       

LagrangeMultipliersRevisedSimplex[vars_,basis_,A_,c_, prResults_:True] :=
Module[{aB,basic=Sort[basis],cB,w},
	aB=Transpose[Transpose[A][[basic]]];
	cB=c[[basic]];
	w=-LinearSolve[Transpose[aB],cB];
	If[prResults,PrintLabelledList[{aB,cB,{w}},{"B",mySubscript["c","B"],
	"Lagrange multipliers"},2]];
	w
]      

CostRangeRevisedSimplex[vars_,basis_,A_,c_,\[CapitalDelta]c_,prResults_:True] :=
Module[{aB,aN,basic=Sort[basis],nonBasic,cB,cN,\[CapitalDelta]cB,\[CapitalDelta]cN,r,rb,w,\[CapitalDelta]w,
	\[Alpha]min=-\[Infinity],\[Alpha]max=\[Infinity],ratios,pos={}, neg={},i=0, varIndices},
	varIndices = Range[Length[vars]]; 
	nonBasic=Complement[varIndices,basic];
	aB=Transpose[Transpose[A][[basic]]];
	aN=Transpose[Transpose[A][[nonBasic]]];
	cB=c[[basic]];cN=c[[nonBasic]];
	\[CapitalDelta]cB=\[CapitalDelta]c[[basic]];\[CapitalDelta]cN=\[CapitalDelta]c[[nonBasic]];
	w=LinearSolve[Transpose[aB],cB];
	\[CapitalDelta]w=LinearSolve[Transpose[aB],\[CapitalDelta]cB];
	r=cN - w.aN;
	rb=\[CapitalDelta]cN - \[CapitalDelta]w.aN;
	ratios = Table[If[rb[[i]]\[NotEqual]0,-r[[i]]/rb[[i]],"-"],
		{i,1,Length[r]}];
	Do[If[MemberQ[nonBasic,j],i=i+1;
		If[rb[[i]]>0,AppendTo[pos,-r[[i]]/rb[[i]]]];
		If[rb[[i]]<0,AppendTo[neg,-r[[i]]/rb[[i]]]]],
		{j,1,Length[vars]}
	];
	If[prResults,
		PrintLabelledList[{\[CapitalDelta]c,r,rb,ratios},
		{"\[CapitalDelta]c","r","rb","-r/rb"},4]
	];
	If[pos!={},\[Alpha]min=Max[pos]];
	If[neg!={},\[Alpha]max=Min[neg]];
	{\[Alpha]min, \[Alpha]max}
]      

rhsRangeRevisedSimplex[vars_,basis_,A_,b_,\[CapitalDelta]b_,prResults_:True] :=
Module[{B,basic=Sort[basis],bbar,\[CapitalDelta]bbar,
	\[Alpha]min=-\[Infinity],\[Alpha]max=\[Infinity],pos={}, neg={},i, ratios},
	B=Transpose[Transpose[A][[basic]]];
	bbar=LinearSolve[B,b];
	\[CapitalDelta]bbar=LinearSolve[B,\[CapitalDelta]b];
	ratios = Table[If[\[CapitalDelta]bbar[[i]]\[NotEqual]0,-bbar[[i]]/\[CapitalDelta]bbar[[i]],"-"],
		{i,1,Length[bbar]}];
	Do[If[\[CapitalDelta]bbar[[i]]>0,AppendTo[pos,-bbar[[i]]/\[CapitalDelta]bbar[[i]]]];
		If[\[CapitalDelta]bbar[[i]]<0,AppendTo[neg,-bbar[[i]]/\[CapitalDelta]bbar[[i]]]],
		{i,1,Length[b]}
	];
	If[prResults,
		PrintLabelledList[{\[CapitalDelta]b,bbar,\[CapitalDelta]bbar,ratios},
		{"\[CapitalDelta]b","bb","\[CapitalDelta]bb","-bb/\[CapitalDelta]bb"},4]
	];
	If[pos!={},\[Alpha]min=Max[pos]];
	If[neg!={},\[Alpha]max=Min[neg]];
	{\[Alpha]min, \[Alpha]max}
]      



End[];

EndPackage[]