(***********************************************************************
This file was generated automatically by the Mathematica front end.
It contains Initialization cells from a Notebook file, which typically
will have the same name as this file except ending in ".nb" instead of
".m".

This file is intended to be loaded into the Mathematica kernel using
the package loading commands Get or Needs.  Doing so is equivalent to
using the Evaluate Initialization Cells menu command in the front end.

DO NOT EDIT THIS FILE.  This entire file is regenerated automatically 
each time the parent Notebook file is saved in the Mathematica front end.
Any changes you make to this file will be overwritten.
***********************************************************************)









BeginPackage["OptimizationToolbox`Chap3Tools`",
"OptimizationToolbox`CommonFunctions`"];

CoFactors::usage = "CoFactors[a], returns matrix of co-factors of square matrix a.";

ShowGradientTangentPlane::usage = "ShowGradientTangentPlane[f, x, y, pt] or ShowGradientTangentPlane[f, x, y, z, pt]. For a function of two or three variables, the function draws a contour passing through the given point and shows gradient vector and tangent plane at the point.";

ShowGradients::usage = "ShowGradients[f,x,y,pts,opts]. For a function of two variables (x,y), the function draws a contour plot and shows gradient vectors at the given points.";

SuperimposedContourPlot::usage = "SuperimposedContourPlot[f1, f2, {x,xmin,xmax}, {y, ymin, ymax}]. The function draws two superimposed contour plots for functions f1 and f2 over the range given.";

TaylorSeriesApprox::usage = "TaylorSeriesApprox[f, vars_List, pt_List], generates linear and quadratic approximations of a function 'f(vars)' around given point 'pt' using the Taylor series. It returns the function value at the given point and its linear and quadratic approximations. For functions of one and two variables, it can also generate a graph comparing approximations with the original function. To get graphs, the arguments must be in the following form.
For f(x): TaylorSeriesApprox[f, {x,xmin,xmax}, pt].
For f(x,y): TaylorSeriesApprox[f, {{x,xmin,xmax},{y,ymin,ymax}}, pt]";

NewtonRaphson::usage = "NewtonRaphson[fcns, vars, start, maxIterations], computes zero of a system of nonlinear functions using the Newton-Raphson method. vars = list of variables, fcns = list of functions, start = starting values of variables, maxIterations = maximum number of iterations allowed (optional, Default is 10).";



Begin["`Private`"];

Needs["Utilities`FilterOptions`"];
Needs["Graphics`ContourPlot3D`"];
Needs["Graphics`PlotField`"];
Needs ["Graphics`Legend`"];



CoFactors[a_]:=Module[
	{n=Length[a],b=a},
		Do[
			Do[b[[i,j]]=(-1)^(i+j) Det[RemoveRowColumn[a,i,j]],{i,1,n}],
			{j,1,n}
		];
		b 
	]		



ShowGradientTangentPlane[f_, x_, y_, pt_]:=Module[{
	vars={x[[1]],y[[1]]},ptRule,gradf,fpt,gradPt,
	grt,grf},
	ptRule=ToRules[vars==pt];
	gradf=Grad[f,vars];
	fpt = f/.ptRule;
	gradPt=gradf/.ptRule;
	tangentPlane=Expand[gradPt.(vars-pt)];
	PrintLabelledList[{f,fpt,gradf,gradPt,tangentPlane==0},
		{"f","f"TextForm[pt],
		"\[Del]f","\[Del]f"TextForm[pt],"TangentPlane"},2];
	grt=ContourPlot[tangentPlane,x,y,
		Contours->{0.},ContourShading -> False,
    	ContourStyle -> {{RGBColor[0,1,0]}},PlotPoints->2,
    	DisplayFunction -> Identity
	];
	grf=ContourPlot[f,x,y,
		Contours->{fpt},ContourShading -> False,
    	ContourStyle -> {{RGBColor[0,0,0]}},PlotPoints->20,
    	DisplayFunction -> Identity
	];
	Show[grf,grt,Graphics[{RGBColor[1,0,0],Line[{pt,
		pt+ (gradPt/Norm[gradPt])}],
		RGBColor[0,0,0],Text["Contour f="<>ToString[fpt],Scaled[{.95,.9}],
		{1,0}],
		RGBColor[0,1,0],Text["Tangent line",Scaled[{.95,.85}],{1,0}],
		RGBColor[1,0,0],Text["Gradient vector",Scaled[{.95,.8}],{1,0}]}],
		PlotRange->{Rest[x],Rest[y]},Axes->True,AxesLabel->vars,
		TextStyle->{FontFamily->"Times",FontSize->10},
		DisplayFunction -> $DisplayFunction]
]

ShowGradientTangentPlane[f_, x_, y_, z_, pt_]:=Module[{
	vars={x[[1]],y[[1]],z[[1]]},ptRule,gradf,fpt,gradPt,
	grt,grf},
	ptRule=ToRules[vars==pt];
	gradf=Grad[f,vars];
	fpt = N[f/.ptRule];
	gradPt=gradf/.ptRule;
	tangentPlane=Expand[gradPt.(vars-pt)];
	PrintLabelledList[{f,fpt,gradf,gradPt,tangentPlane==0},
		{"f","f"TextForm[pt],
		"\[Del]f","\[Del]f"TextForm[pt],"TangentPlane"},2];
	grt=ContourPlot3D[tangentPlane,x,y,z,
		Contours->{0.},Lighting -> False,
		ContourStyle -> {{RGBColor[0,1,0]}},
		PlotPoints->{2,2},
    	DisplayFunction -> Identity
	];
	grf=ContourPlot3D[f,x,y,z,
		Contours->{fpt},Lighting -> False,
    	ContourStyle -> {{GrayLevel[0.5]}},PlotPoints->{5,4},
    	DisplayFunction -> Identity
	];
	Show[grf,grt,Graphics3D[{RGBColor[1,0,0],Line[{pt,
		pt+ (gradPt/Norm[gradPt])}],
		RGBColor[0,0,0],Text["Contour f="<>ToString[fpt],Scaled[{.95,.9,.9}],
		{1,0}],
		RGBColor[0,1,0],Text["Tangent plane",Scaled[{.95,.85,.85}],{1,0}],
		RGBColor[1,0,0],Text["Gradient vector",Scaled[{.95,.8,.8}],{1,0}]}],
		PlotRange->{Rest[x],Rest[y],Rest[z]},
		Axes->True,AxesLabel->vars,
		TextStyle->{FontFamily->"Times",FontSize->10},
		Boxed->False,DisplayFunction -> $DisplayFunction]
]



ShowGradients[f_,x_, y_,pts_List,opts___]:=Module[{ptsRules,fpts,gradf,gradPts,vecs,a,b},
	gradf={D[f,x],D[f,y]};
	ptsRules=Map[ToRules[{x,y}==#]&,pts];
	fpts=f/.ptsRules;
	gradPts=gradf/.ptsRules;
	vecs=Table[{pts[[i]],gradPts[[i]]},{i,1,Length[pts]}];
	a=	LabelContourLines[ContourPlot[f, x,y,
		Contours->Sort[fpts],Axes->True,
		AxesLabel->{x,y},
		ContourShading->False,
		DisplayFunction -> Identity,opts],
		DisplayFunction -> Identity
	];
	b=ListPlotVectorField[vecs,ScaleFactor -> .5,
		DisplayFunction -> Identity
	];
	Show[{a,b},DisplayFunction -> $DisplayFunction]
]



SuperimposedContourPlot[f1_,f2_,x_List,y_List]:=Module[{a,b},
	a=ContourPlot[f1,x,y,ContourShading->False,
		DisplayFunction->Identity];
	b=ContourPlot[f2,x,y,Axes->True,
		ContourShading->False,ContourStyle->{Hue[.5]},
		DisplayFunction->Identity];
	Show[{a,b},Axes->True,AxesLabel->{x[[1]],y[[1]]},
		DisplayFunction->$DisplayFunction]
];



TaylorSeriesApprox[f_, {x_List,y_List}, pt_List, opts___?OptionQ]:=
Module[{vars={x[[1]],y[[1]]}, fpt, lin, quad, grf, grLin, grQuad},
	{fpt, lin, quad} = TaylorSeriesApprox[f, vars, pt];
	grLin=ContourPlot[lin, x,y,
		Contours->{fpt},ContourShading -> False,
    		ContourStyle -> {{RGBColor[1,0,0]}},PlotPoints->2,
    		DisplayFunction -> Identity
		];
	grQuad=ContourPlot[quad, x,y,
		Contours->{fpt},
			Evaluate[FilterOptions[ContourPlot,Flatten[{opts}]]],
			ContourShading -> False,
    		ContourStyle -> {{RGBColor[0,1,0]}},PlotPoints->10,
    		DisplayFunction -> Identity
		];
	grf=ContourPlot[f, x,y,
		Contours->{fpt},
		Evaluate[FilterOptions[ContourPlot,Flatten[{opts}]]],
		ContourShading -> False,
    		ContourStyle -> {{RGBColor[0,0,0]}},PlotPoints->20,
    		DisplayFunction -> Identity
		];
	Show[grf,grLin,grQuad,Graphics[{
		RGBColor[0,0,0],Text["Contour f="<>ToString[fpt],Scaled[{.95,.9}],
		{1,0}],
		RGBColor[1,0,0],Text["Lin. approx",Scaled[{.95,.85}],{1,0}],
		RGBColor[0,1,0],Text["Quad. approx",Scaled[{.95,.8}],{1,0}]}],
		Axes->True, AxesLabel->vars,
		TextStyle->{FontFamily->"Times",FontSize->10},
		DisplayFunction -> $DisplayFunction];
	{fpt, lin, quad}
];


TaylorSeriesApprox[f_, vars_List, pt_List]:=
Module[{gradf,fpt,gradPt,h,hpt, lin, quad, \[CapitalDelta]x},
	gradf=Simplify[Grad[f,vars]];
	h=Simplify[Hessian[f,vars]];
	{fpt,gradPt,hpt} = {f,gradf,h}/.ToRules[vars==N[pt]];
	\[CapitalDelta]x = vars-pt;
	lin = fpt + Expand[gradPt.\[CapitalDelta]x];
	quad = Expand[lin + 1/2 \[CapitalDelta]x.hpt.\[CapitalDelta]x];
	PrintLabelledList[{f,fpt,gradf,gradPt,h,hpt,{\[CapitalDelta]x}},
		{"f","f"TextForm[pt],
		"\[Del]f","\[Del]f"TextForm[pt],
		"\!\(TraditionalForm\`\[Del]\^2\)f","\!\(TraditionalForm\`\[Del]\^2\)f"TextForm[pt],"\[CapitalDelta]x"},2];
	PrintLabelledList[{lin,quad},{
		"Linear approximation","Quadratic approximation"},1];
	{fpt, lin, quad}
];

TaylorSeriesApprox[f_, x_, pt_]:=TaylorSeriesApprox[f, {x}, {pt}];
TaylorSeriesApprox[f_, x_List, pt_]:=
Module[{fpt, lin, quad, grf, grLin, grQuad},
	{fpt, lin, quad} = TaylorSeriesApprox[f, {x[[1]]}, {pt}];
	Plot[{lin,quad,f}, x,
		PlotStyle\[Rule]{RGBColor[1,0,0],RGBColor[0,1,0],RGBColor[0,0,0]},
	PlotLegend -> {"Lin.","Quad.","f"},
	LegendPosition->{1,0},TextStyle->{FontFamily->"Times",FontSize->10}];
	{fpt, lin, quad}
]



NewtonRaphson[fcns_, vars_, start_, maxIterations__:10]:=NewtonRaphson[{fcns}, {vars}, {start}, maxIterations];

NewtonRaphson[fcns_List, vars_List, start_List, maxIterations__:10]:=
Module[{jacobian, xk, fk, jk, jki, deltaxk, normfk, tol=0.001, i=0,hist={}},
	jacobian = JacobianMatrix [fcns, vars];
	Print["Jacobian matrix"->MatrixForm[jacobian]]; 
	xk = N[start];
	fk = N[fcns /. ToRules[vars==xk]];
	normfk = Norm[fk];
	hist={xk,normfk};
	While [(normfk > tol) && (i < maxIterations), 
	  i = i + 1;
	  fk = N[fcns /. ToRules[vars==xk]];
	  jk = N[jacobian /. ToRules[vars==xk]];
	  jki = Inverse[jk];
	  deltaxk = -jki.fk;
	  normfk = Norm[fk];
	  Print["*********** Iteration "<>ToString[i]<>" ***********"];
	  PrintLabelledList[{xk,fk,normfk},
	  {"xk","F(xk)","||F(xk)||"},3];
	  PrintLabelledList[{jk, jki},
	  	{"J(xk)","Inverse[J(xk)]"},1];
	  xk = xk + deltaxk;
	  hist={hist,xk,normfk};
	  PrintLabelledList[{deltaxk,xk},{"\[CapitalDelta]x", "New xk"},2];
	];
	If[normfk>tol,Print["No convergence yet"]];
	{xk,Partition[Flatten[hist],1+Length[vars]]}
]

JacobianMatrix [f_, vars_] := Map[dfi[#,vars]&,f];
dfi[fi_, vars_] := Map[D[fi,#]&,vars];

Norm[f_] := N[Sqrt[Apply[Plus,Map[#^2&,f]]]];



End[];

EndPackage[]