(***********************************************************************
This file was generated automatically by the Mathematica front end.
It contains Initialization cells from a Notebook file, which typically
will have the same name as this file except ending in ".nb" instead of
".m".

This file is intended to be loaded into the Mathematica kernel using
the package loading commands Get or Needs.  Doing so is equivalent to
using the Evaluate Initialization Cells menu command in the front end.

DO NOT EDIT THIS FILE.  This entire file is regenerated automatically 
each time the parent Notebook file is saved in the Mathematica front end.
Any changes you make to this file will be overwritten.
***********************************************************************)









BeginPackage["OptimizationToolbox`InteriorPoint`",
"OptimizationToolbox`CommonFunctions`"];



FormDualLP::usage = "FormDualLP[f, g, vars, options]. Forms dual of the given LP. f is the objective function, g is a list of constraints, and vars is a list of variables. See Options[FormDualLP] to find out about a list of valid options for this function.";

Options[FormDualLP] =  
	{UnrestrictedVariables->{},
	ProblemType->Min, StandardVariableName->"x",
	DualLPVariableName->"v"};

DualLPVariableName::usage="DualLPVariableName is an option for dual interior point methods. It defines the symbol to be used when creating the dual variable names (the Lagrange multipliers for equality constraints). The default is v.";



PrimalAffineLP::usage = "PrimalAffineLP[f, g, vars, options]. Solves an LP problem using the primal affine algorithm. f is the objective function, g is a list of constraints, and vars is a list of variables. See Options[PrimalAffineLP] to find out about a list of valid options for this function.";

Options[PrimalAffineLP] =  
	{UnrestrictedVariables->{}, MaxIterations->20,
	ProblemType->Min, StandardVariableName->"x",
	PrintLevel->1,
	StepLengthFactor->0.99,
	ConvergenceTolerance->{0.001,0.2,2,0.5},
	StartingVector->{}};

StepLengthFactor::usage="StepLengthFactor is an option for interior point methods. It is the reduction factor applied to the computed step length to maintain feasibility. The default is 0.99";

StartingVector::usage="StartingVector is an option for several interior point methods. The default is {1,\[Ellipsis],1}.";



PrimalDualLP::usage = "PrimalDualLP[f, g, vars, options]. Solves an LP problem using the interior point algorithm based on solving KT conditions using the Newton-Raphson method. f is the objective function, g is a list of constraints, and vars is a list of variables. See Options[PrimalDualLP] to find out about a list of valid options for this function.";

Options[PrimalDualLP] =  
	{UnrestrictedVariables->{}, MaxIterations->20,
	ProblemType->Min, StandardVariableName->"x",
	PrintLevel->1,
	StepLengthFactor->0.99,
	ConvergenceTolerance->0.0001,
	StartingVector->{}};



Begin["`Private`"];

Needs["Utilities`FilterOptions`"];





InteriorPtStandardLP[type_, f_, g_, designVars_, 
unrestrictedVars_, varName_, prResults_:True] :=
Module[{n, nf = f, ng = g, h={}, con, vars},
	n=Length[designVars]+Length[unrestrictedVars];
	vars = Table[mySubscript[varName,i], {i,1,n}];
	If[type===Max, nf = -f];
	nf = InteriorPtUnrestricted[nf, designVars, unrestrictedVars, vars];
	ng = Map[StandardizeConstraint[#,designVars]&,g];
	ng = Map[InteriorPtUnrestricted[#, designVars, 
		unrestrictedVars, vars]&,ng];
	Do[	con=ng[[i]];
		Switch[Head[con],
			LessEqual, newVar=mySubscript[varName,Length[vars]+1];
				AppendTo[h,First[con] + newVar==Last[con]];
				AppendTo[vars,newVar],
			GreaterEqual, newVar=mySubscript[varName,Length[vars]+1];
				AppendTo[h,First[con] - newVar==Last[con]];
				AppendTo[vars,newVar],
			Equal,
				AppendTo[h,con]
		],{i,1,Length[ng]}
	];
	If[prResults,
		Print["Minimize ", nf];
		Print["Subject to ", MatrixForm[h]];
		Print["All variables \[GreaterEqual] 0"]
	];
	{nf, h, vars}
]       

InteriorPtUnrestricted[fcn_, vars_, unrestrictedVars_, x_] := 
Module[{xv={},p=1,i},
	Do[If[MemberQ[unrestrictedVars,i],
			AppendTo[xv,{vars[[i]]->x[[p]]-x[[p+1]]}];
			p=p+2,
			AppendTo[xv,{vars[[i]]->x[[p]]}];
			p=p+1
		],{i,1,Length[vars]}
	];		
	Map[Expand,fcn /. Flatten[xv]]
]

getConstant[poly_, vars_] := Map[Fold[Coefficient[#1, #2, 0]&,#,vars]&, poly]; 

getCoefficients[poly_, vars_] := Map[Coefficient[poly,#]&, vars]; 

StandardizeConstraint[g_, vars_] :=
Module[{a,b, ng},
	Switch[Head[g],
		LessEqual, poly = g[[1]] - g[[2]];
			a = getCoefficients[poly,vars];
			b = getConstant[poly, vars];
			If[b<=0, ng = a.vars <= -b, ng = -a.vars >= b],
		GreaterEqual, poly = g[[1]] - g[[2]];
			a = getCoefficients[poly,vars];
			b = getConstant[poly, vars];
			If[b<=0, ng = a.vars >= -b, ng = -a.vars <= b],
		Equal, poly = g[[1]] - g[[2]];
			a = getCoefficients[poly,vars];
			b = getConstant[poly, vars];
			If[b<=0, ng = a.vars == -b, ng = -a.vars == b]
	];
	ng
]

compUp[A_,p_,PrintResults_]:=Module[{ap,P},
	ap=A.DiagonalMatrix[p];
	P=Transpose[ap].PseudoInverse[ap.Transpose[ap]].ap;
	If[PrintResults,
		Print["p"->p];
		PrintLabelledList[{A,P},
		{"A","P"},1]
	];
	Table[P[[i,i]],{i,1,Length[p]}]
];
VertexSolutionPAS[A_,b_,x_,vars_,\[Epsilon]_, prResults_]:=
Module[{n=Length[A],j=0,P,p=x,basic={},nonBasic={},xz,status,sol,
	pp},
	If[prResults,
		StylePrint["***** Finding nearest vertex solution *****",
		"Output",FontWeight->"Bold"]
	];
	While[Length[basic]<n,
		++j;
		If[prResults,Print["Pass"->j]];
		p=compUp[A,p,prResults];
		pp = Select[p,#\[GreaterEqual](1-\[Epsilon])&];
		basic=Flatten[Union[Map[Position[p,#]&,pp]]];
	];
	basic = vars[[basic]];
	nonBasic = Complement[vars, basic];
	xz=Map[#->0&,nonBasic];
	sol=Sort[Flatten[{Solve[(A.(vars/.xz))==b],xz}]];
	If[Min[vars/.sol]\[GreaterEqual]0.,status="Feasible",status="Infeasible"];	
	{sol,status}
];



FormDualLP[f_, g_, designVars_,opts___?OptionQ]:=
Module[{n,freeIndices,unrestrictedVars, type, varName, u,v,du,dv,
	vars, sf, sg,c, A, b, df, rhs},
	{unrestrictedVars, type, varName, v}= 
		{UnrestrictedVariables, ProblemType,
		StandardVariableName, DualLPVariableName} /. {opts} /. 
		Options[FormDualLP];
	freeIndices = Sort[Flatten[Map[
		Position[designVars,#]&,unrestrictedVars]]];
	StylePrint["Primal problem",FontWeight->"Bold"];
	{sf, sg,vars} = InteriorPtStandardLP[type, f, Flatten[g],
		designVars,freeIndices, varName];
	n=Length[vars];
	c = getCoefficients[sf, vars]; 
	b=Map[Last[#]&,sg];
	A=Map[Coefficient[First[#],vars]&,sg];
	PrintLabelledList[{A,b,c},{"A","b","c"},3];
	dv = Table[mySubscript[v,i],{i,1,Length[b]}];
	df = Expand[dv.b];
	dg = Thread[(c - Transpose[A].dv)\[GreaterEqual]0];
	StylePrint["Dual LP problem", FontWeight->"Bold"];
	PrintLabelledList[{{dv},df,dg},
	{"Variables","Maximize","Subject to "},1];
	{df,dg,dv}
];



PrimalAffineLP[f_, g_, designVars_,opts___?OptionQ]:=
Module[{n, freeIndices, unrestrictedVars, maxIter, type, varName,
	prLevel, prSetup, prResults, \[Beta], tol, vars, 
	sf, sg, sol={}, fstar={}, varsMapping, k, i, convgParam,
	c, A, A1,b, iter=0, status="NonOptimum",x,c1,x1,phase,history={}},
	{unrestrictedVars, maxIter, type, varName, prLevel,
		\[Beta],tol,x} = 
		{UnrestrictedVariables, MaxIterations, ProblemType,
		StandardVariableName, PrintLevel,
		StepLengthFactor,ConvergenceTolerance,
		StartingVector} /. Flatten[{opts, Options[PrimalAffineLP]}];
	Switch[prLevel,
		0, {prSetup, prResults}={False, False}, 
		1, {prSetup, prResults}={True, False}, 
		_, {prSetup, prResults}={True, True}
	];
	freeIndices = Sort[Flatten[Map[
		Position[designVars,#]&,unrestrictedVars]]];
	{sf, sg,vars} = InteriorPtStandardLP[type, f, g,
		designVars,freeIndices, varName, prSetup];
	varsMapping = {};
	k = 1;
	Do[If[FreeQ[freeIndices,i],
		varsMapping={varsMapping,designVars[[i]]->vars[[k]]};
		k = k + 1,
		varsMapping={varsMapping,
			designVars[[i]]->vars[[k]]-vars[[k+1]]};
		k = k + 2],
		{i,1,Length[designVars]}
	];
	varsMapping=Flatten[varsMapping];
    If[prSetup,Print["Problem variables redefined as:",varsMapping]];
	n=Length[vars];
	If[x==={},x=Table[1.0,{n}]];
	AppendTo[history,x];
	c=Coefficient[sf,vars];
	b=Map[Last[#]&,sg];
	A=Map[Coefficient[First[#],vars]&,sg];
	x1=Join[x,{1}];
	If[prResults,
		PrintLabelledList[{c,b,A},{"b","c","A"},2];
		Print["Starting point"->x,
			" Objective function"->sf/.ToRules[vars==x],
			" Status"->status]
	];
	(* Phase I *)
	If[Norm[b-A.x]/(Norm[b]+1)>tol[[2]],
		While[And[status==="NonOptimum",iter<maxIter],
			++iter;
			phase = 1;
			A1=Transpose[Join[Transpose[A],{b-A.x}]];
			c1=Join[Table[0,{n}],{Last[x1]}];
			If[prResults,
				StylePrint["***** Iteration "<>ToString[iter]<>
				" (Phase "<>ToString[phase]<>") *****",
					"Output",FontWeight->"Bold"]
			];
			{x1,status,convgParam}=PrimalAffineLPStep[
				n+1,A1,b,c1,x1,\[Beta],tol,prResults];
			x=Drop[x1,-1];
			AppendTo[history,x];
			If[prResults,
				Print["New point"->x,
				" Artificial objective function"->Last[x1],
				" Objective function"->sf/.ToRules[vars==x],
				" Status"->status]
			]
		]
	];
	If[And[iter<maxIter,Norm[b-A.x]/(Norm[b]+1)>tol[[2]]],
		Print["Phase I ended with no feasible solution"];
		Print["New point"->x];
		Print["Convergence parameters"->convgParam];
		Print["Constraint values norm"->Norm[b-A.x]/(Norm[b]+1)];
		Abort[];
	];
	(* Phase II *)
	status = "NonOptimum";
	While[And[status==="NonOptimum",iter<maxIter],
		++iter;
		phase = 2;
		If[prResults,
			StylePrint["***** Iteration "<>ToString[iter]<>
				" (Phase "<>ToString[phase]<>") *****",
				"Output",FontWeight->"Bold"]
		];
		{x,status,convgParam}=PrimalAffineLPStep[
			n,A,b,c,x,\[Beta],tol,prResults];
		AppendTo[history,x];
		If[prResults,
			Print["New point"->x,
			" Objective function"->sf/.ToRules[vars==x],
			" Status"->status]
		];
	];
	(* Purification phase *)
	sol = varsMapping /. ToRules[vars==x];
	If[prSetup,
		StylePrint["***** "<>ToString[status]<>" solution after "<>
		ToString[iter]<>" iterations *****",
		"Output",FontWeight->"Bold"];
		Print["Interior solution"->sol," Objective function"->f/.sol];
		Print["Convergence parameters"->convgParam]
	];
	If[status==="Optimum",
		{sol,status} = VertexSolutionPAS[A,b,x,vars,tol[[1]],prResults];
		sol = varsMapping /. sol;
		fstar = f/.sol;
		If[prSetup,
			Print["Nearest vertex solution"->sol,
			" Objective function"->fstar," Status"->status]
		];
	];
	{{fstar, sol}, history}
];

PrimalAffineLPStep[n_,A_,b_,c_,x_,\[Beta]_,\[Epsilon]_, prResults_]:=
Module[{newx=x,Tk=DiagonalMatrix[x],w,ATk,AT=Transpose[A],
	status="NonOptimum",r,\[Alpha]=0,lhs, rhs,d,\[Sigma]1,\[Sigma]2,\[Sigma]3,\[Sigma]4},
	ATk=A.(Tk^2);
	lhs = ATk.AT; rhs = ATk.c;
	w=LinearSolve[lhs, rhs];
	r=c-AT.w;
	d=-Tk.r;
	\[Sigma]1=Norm[d];
	\[Sigma]2=Norm[(A.x)-b]/(Norm[b]+1);
	\[Sigma]3=Norm[r]/(Norm[c]+1);
	\[Sigma]4=Abs[c.x - b.w];
	If[Apply[And,Flatten[{Thread[{\[Sigma]1,\[Sigma]2,\[Sigma]3,\[Sigma]4}\[LessEqual]\[Epsilon]],Min[x]\[GreaterEqual]0.}]],
	 status="Optimum",
		If[Apply[And,Map[NonNegative,Chop[d]]],status="Unbounded",
			\[Alpha] = \[Beta] Min[Select[-1/d,Positive]];
			newx=x+\[Alpha] Tk.d
		]
	];
	If[prResults,
		Print["Tk[diagonal]"->x];
		PrintLabelledList[{lhs},{"A.Tk.Tk.AT"},1];
		PrintLabelledList[{rhs, w, r, d},
		{"A.Tk.Tk.c","w","r","d"},2];
		Print["Convergence parameters"->{\[Sigma]1,\[Sigma]2,\[Sigma]3,\[Sigma]4},
			" \[Beta](-1/d)"->-\[Beta]/d," Step length, \[Alpha]"->\[Alpha]]
	];
	{newx,status,{\[Sigma]1,\[Sigma]2,\[Sigma]3,\[Sigma]4}}
];



PrimalDualLP[f_, g_, designVars_,opts___Rule]:=
Module[{m, n, freeIndices, unrestrictedVars, maxIter, type, varName,
	prLevel, prSetup, prResults, \[Beta], tol, vars, 
	sf, sg, sol={}, fstar={}, startVec, varsMapping, i, k,
	c, A, b, iter=0, status="NonOptimum", xk, uk, vk, history={}},
	{unrestrictedVars, maxIter, type, varName, prLevel,
		\[Beta],tol,startVec} = 
		{UnrestrictedVariables, MaxIterations, ProblemType,
		StandardVariableName, PrintLevel,
		StepLengthFactor,ConvergenceTolerance,
		StartingVector} /. Flatten[{opts, Options[PrimalDualLP]}];
	Switch[prLevel,
		0, {prSetup, prResults}={False, False}, 
		1, {prSetup, prResults}={True, False}, 
		_, {prSetup, prResults}={True, True}
	];
	freeIndices = Sort[Flatten[Map[
		Position[designVars,#]&,unrestrictedVars]]];
	{sf, sg,vars} = InteriorPtStandardLP[type, f, g,
		designVars,freeIndices, varName, prSetup];
	varsMapping = {};
	k = 1;
	Do[If[FreeQ[freeIndices,i],
		varsMapping={varsMapping,designVars[[i]]->vars[[k]]};
		k = k + 1,
		varsMapping={varsMapping,
			designVars[[i]]->vars[[k]]-vars[[k+1]]};
		k = k + 2],
		{i,1,Length[designVars]}
	];
	varsMapping=Flatten[varsMapping];
    If[prSetup, Print["Problem variables redefined as:",varsMapping]];
	c=Coefficient[sf,vars];
	b=Map[Last[#]&,sg];
	A=Map[Coefficient[First[#],vars]&,sg];
	m=Length[b];
	n=Length[vars];
	If[startVec==={},xk=Table[1.0,{n}];uk=xk;vk=Table[0.0,{m}],
		{xk,uk,vk}=startVec];
	If[prResults,
		PrintLabelledList[{b,c,A},{"b","c","A"},2];
		Print["**** Starting vectors ****"];
		Print["Primal vars (x)"->xk];
		Print["Dual vars (u)"->uk];
		Print["Multipliers (v)"->vk];
		PrintLabelledList[{sf/.ToRules[vars==xk],status},
			{"Objective function","Status"}]
	];
	status = "NonOptimum";
	AppendTo[history,xk];
	While[And[status==="NonOptimum",iter<maxIter],
		++iter;
		If[prResults,
			StylePrint["***** Iteration "<>ToString[iter]<>" *****",
			"Output",FontWeight->"Bold"];
		];
		{xk,uk,vk,status}=PrimalDualLPStep[
			iter, A,b,c,xk,uk,vk,\[Beta],tol,prResults];
		AppendTo[history,xk];
		If[prResults,
			PrintLabelledList[{sf/.ToRules[vars==xk],status},
			{"Objective function","Status"}]
		];
	];
	(* Purification phase *)
	sol = varsMapping /. ToRules[vars==xk];
	If[prSetup,
		StylePrint["***** "<>ToString[status]<>" solution after "<>
		ToString[iter]<>" iterations *****","Output",FontWeight->"Bold"];
		Print["Interior solution"->sol];
		Print["Objective function"->f/.sol];
	];
	If[status==="Optimum",
		{sol, status} = VertexSolutionPAS[A,b,xk,vars,tol,prResults];
		If[status!="NotFound",
		 sol = varsMapping /. sol;
		 fstar = f/.sol; 
			If[prSetup,
			 Print["Nearest vertex solution"->sol];
			 Print["Objective function"->fstar];
			 Print["Status"->status]
			]
		]
	];
	{{fstar, sol}, history}
];

PrimalDualLPStep[k_,A_,b_,c_,xk_,uk_,vk_,\[Beta]_,\[Epsilon]_,PrintResults_]:=
Module[{newx=xk,newu=uk, newv=vk,D,AD,AT=Transpose[A],
	status="NonOptimum",rp,rc,rd,\[Alpha]p=0,\[Alpha]d=0,\[Sigma]p,\[Sigma]d,dv=0,du=0,dx=0,
	\[Mu],n=Length[xk],ADAT, rhs, d=xk/uk, dxk, duk},
	D = DiagonalMatrix[d];
	\[Mu]=((xk.uk)/n)/k;
	rp=-(A.xk)+b;
	rd= -(AT.vk)-uk+c;
	rc=-uk+\[Mu](1/xk);
	\[Sigma]p=Norm[rp]/(Norm[b]+1);
	\[Sigma]d=Norm[rd]/(Norm[c]+1);
	If[PrintResults,
		Print["D[diagonal]"->d];
		PrintLabelledList[{rp, rd, rc},
			{mySubscript["r","p"],
			mySubscript["r","d"],mySubscript["r","c"]},3];
		PrintLabelledList[{\[Sigma]p, \[Sigma]d, \[Mu]},
			{"Parameters: \[Sigma]p", "\[Sigma]d", "\[Mu]"}]
	];

	If[Max[\[Sigma]p,\[Sigma]d,\[Mu]]\[LessEqual]\[Epsilon], status="Optimum",
		AD=A.D;
		ADAT=AD.AT;
		rhs = rp+(AD.(-rc+rd));
		dv=LinearSolve[ADAT,rhs];
		du=-(AT.dv)+rd;
		dx=D.(-du+rc);
		If[And[Abs[Max[rp]]\[LessEqual]\[Epsilon],Min[dx]>0,c.dx<0],status="Unbounded"];
		If[And[Abs[Max[rd]]\[LessEqual]\[Epsilon],Min[du]>0,b.dv>0],status="Unbounded"];
		dxk = \[Beta] Table[If[dx[[i]]<0,-xk[[i]]/dx[[i]],\[Infinity]],{i,1,Length[dx]}];
		duk = \[Beta] Table[If[du[[i]]<0,-uk[[i]]/du[[i]],\[Infinity]],{i,1,Length[du]}];
		\[Alpha]p = Min[1, dxk];
		\[Alpha]d = Min[1, duk];
		newx=xk+\[Alpha]p dx;
		newu=uk+\[Alpha]d du;
		newv=vk+\[Alpha]d dv;
		If[PrintResults,
			PrintLabelledList[{ADAT},{"A.D.AT"},1];
			Print["rp+A.D(-rc+rd)"->rhs];
			Print["dv"->dv];
			Print["du"->du];
			Print["dx"->dx];
			Print["-\[Beta]x/dx"->dxk];
			Print["-\[Beta]u/du"->duk];
			PrintLabelledList[{\[Alpha]p,\[Alpha]d},
			{mySubscript["\[Alpha]","p"], mySubscript["\[Alpha]","d"]},2];
			Print["New primal vars (x)"->newx];
			Print["New dual vars (u)"->newu];
			Print["New multipliers (v)"->newv];
		];
	];
	{newx,newu,newv,status}
];



End[];

EndPackage[]