(***********************************************************************
This file was generated automatically by the Mathematica front end.
It contains Initialization cells from a Notebook file, which typically
will have the same name as this file except ending in ".nb" instead of
".m".

This file is intended to be loaded into the Mathematica kernel using
the package loading commands Get or Needs.  Doing so is equivalent to
using the Evaluate Initialization Cells menu command in the front end.

DO NOT EDIT THIS FILE.  This entire file is regenerated automatically 
each time the parent Notebook file is saved in the Mathematica front end.
Any changes you make to this file will be overwritten.
***********************************************************************)









BeginPackage["OptimizationToolbox`OptimalityConditions`",
"OptimizationToolbox`CommonFunctions`"];



UnconstrainedOptimality::usage = "UnconstrainedOptimality[f, vars, opts], finds all stationary points of function f of vars. Using second-order sufficient conditions, it then classifies them into minimum, maximum or inflection points. Options can be used to specify a Mathematica function to be used for solving the resulting system of equations. If this function itself has options, they can also be specified."; 

Options[UnconstrainedOptimality] = {PrintLevel->1,SolveEquationsUsing->NSolve,
	StartingSolution->{}};

SolveEquationsUsing::usage="SolveEquationsUsing is an option for Optimality-condition based methods. The Mathematica function used to solve system of equations is specified with this option. The default is NSolve. If using FindRoot, a starting solution must be specified using the StartingSolution option."; 

StartingSolution::usage="The StartingSolution to be used with the FindRoot function in Mathematica. It is used only if the method specified is FindRoot."; 



KTSolution::usage="KTSolution[f, con, vars, opts], finds candidate minimum points by solving KT conditions. f is the objective function. con is a list of constraints. The function automatically converts the constraints to standard form before proceeding. vars is a list of problem variables.  Several options can be used with the function. See Options[KTSolution]. By default, all possible cases of inequalities being active are examined. Active inequalities can be explicitly specified through the ActiveCases option.";

Options[KTSolution] = {PrintLevel->1,
	ActiveCases->{},KTVarNames->{"u","s","v"}, SolveEquationsUsing->NSolve,
	StartingSolution->{}};

ActiveCases::usage="ActiveCases->List of active LE constraints to be considered in the solution. The default is to consider all possible cases.";

KTVarNames::usage="KTVarNames->Variable names used for g ('\[LessEqual]' constraints) multipliers, slack variables and h ('=' constraints) multipliers. Default is {u,s,v}."; 

NewFUsingSensitivity::usage = "NewFUsingSensitivity[vars, f, sol, g, mg] returns the new value of the objective function when constants in one or more constraints are changed. vars is a list of optimization variables, f is the objective function, sol is the list of optimum values of variables followed by the Lagrange multipliers, g is a list of original constraints, and mg is a list of modified constraints.";



ConstrainedSufficiency::usage="ConstrainedSufficiency[f,con,vars,ktsoln,opts], performs computations necessary for checking second order sufficient conditions for constrained problems. It forms the hessian of the Lagrangian (HL), determines the feasible changes (d), and returns d.HL.d. f is the objective function. con is a list of constraints. vars is a list of problem variables. ktsoln contain solution point and Lagrange multipliers obtained from KT solution.  Several options can be used with the function. See Options[ConstrainedSufficiency].";

Options[ConstrainedSufficiency] = {PrintLevel->1, FeasibleChangesName->"d"};

FeasibleChangesName::usage="FeasibleChangesName->Variable names used for feasible changes. Default is 'd'.";



ConstructDual::usage="ConstructDual[f,con,vars,opts], constructs dual of the given problem. Several options can be used with the function. See Options[ConstructDual].";

Options[ConstructDual] = {PrintLevel->1, DualVariableNames->{"U","V"}};

DualVariableNames::usage="DualVariables->Variable names used for dual variables. Default is {U,V}.";



Begin["`Private`"];

Needs["Utilities`FilterOptions`"];
Needs["DiscreteMath`Combinatorica`"];





UnconstrainedOptimality[f_, vars_, opts___?OptionQ] := 
Module[{grad, solns, hessian, eqns, method, strtSoln, prLevel, prResults},
	{prLevel, method, strtSoln} = 
		{PrintLevel, SolveEquationsUsing,StartingSolution}/.
		Flatten[{opts, Options[UnconstrainedOptimality]}];
	If[prLevel>0, prResults=True, prResults=False];
	grad = Grad[f, vars]; 
	hessian = Hessian[f, vars]; 
	eqns = Map[#==0&,grad];
	If[method===FindRoot,
		solns = FindRoot[eqns, 
			Evaluate[Apply[Sequence,Transpose[{vars,strtSoln}]]], 
			Evaluate[FilterOptions[FindRoot,opts]],Compiled->False];
			If[Head[solns]===method,solns={},solns={solns}],
		solns = method[eqns, vars,Evaluate[FilterOptions[method,opts]]]
	]; 
	solns = Union[Select[solns, FreeQ[#1, _Complex, 2] & ]];
	nm[x_]:=Apply[And,Map[NumericQ,vars/.x]];
	solns = Select[solns,nm]; 
	If[prResults,
		PrintLabelledList[{f,grad,hessian},{"Objective function",
		"Gradient vector","Hessian matrix"},1];
		Print["****** First-order optimality conditions ******"]; 
		PrintLabelledList[{eqns},{"Necessary conditions"},1];
		PrintLabelledList[{solns},
		{"Possible solutions (stationary points)"},1];
		Print["****** Second-order optimality conditions ******"]; 
	];
	UnconstrainedSufficientCheck[f,hessian,vars,#1,prResults]& /@ solns 
]; 

UnconstrainedSufficientCheck[f_, hessian_, vars_, point_, prResults_] := 
   Module[{H, pm, oddPM, evenPM, status = "InflectionPoint"}, 
    H = hessian /. point;
    pm = PrincipalMinors[H]; 
	oddPM = pm[[Select[Range[Length[pm]], OddQ]]]; 
	evenPM = pm[[Select[Range[Length[pm]], EvenQ]]];
	Which[
		Not[Apply[And,Map[NumericQ,pm]]], status = "Undetermined",  
		Apply[Or,Map[N[Chop[#]]==0.&,pm]], status = "Undetermined", 
		Apply[And,Map[Positive,pm]], status = "MinimumPoint", 
		Apply[And,Map[Negative,oddPM]] && Apply[And,Map[Positive,evenPM]],
		 status = "MaximumPoint"
	]; 
	If[prResults,
		Print["------- Point"->point];
		If[Length[vars]\[GreaterEqual]4,
			PrintLabelledList[{H,pm},
			{"Hessian","Principal minors"},1]; 
			PrintLabelledList[{status,f/.point},
			{"Status","Function value"},2], 
			PrintLabelledList[{H,pm,status,f/.point},
			{"Hessian","Principal minors","Status","Function value"},2]
		]
	]; status->point
]; 



KTSolution[f_,con_List,vars_List,opts___?OptionQ]:=
Module[{u,s,v,uVars={},sVars={},vVars={},
	fLabel="f",gLabels={},hLabels={}, g={}, h={},
	dg={}, dh={}, dgLabels={}, dhLabels={},
	prLevel, prSetup, prResults, active, soln, solnPts, fStar, L, eqns},
	{prLevel,active,{u,s,v}} = {PrintLevel,ActiveCases,
		KTVarNames} /. {opts} /. Options[KTSolution];
	Switch[prLevel,
		0, {prSetup, prResults}={False, False}, 
		1, {prSetup, prResults}={True, False}, 
		_, {prSetup, prResults}={True, True}
	]; 
	If[prSetup,
		PrintLabelledList[{f,Grad[f,vars]},{"Minimize f","\[Del]f"},1]
	];
	{g,h} = StandardConstraints[con];
	If[g!={},
		gLabels = Table[mySubscript["g",ToString[i]],{i,1,Length[g]}];
		dgLabels = Table[mySubscript["\[Del]g",ToString[i]],{i,1,Length[g]}];
		dg=Grad[g,vars];
		uVars=Table[mySubscript[u,i],{i,1,Length[g]}];
		sVars=Table[mySubscript[s,i]^2,{i,1,Length[g]}];
		If[prSetup,
			Print["***** LE constraints and their gradients"];
			PrintLabelledList[Join[Thread[g\[LessEqual]0],dg],
			Join[gLabels,dgLabels],2]
		]
	];
	If[h!={},
		hLabels = Table[mySubscript["h",ToString[i]],{i,1,Length[h]}];
		dhLabels = Table[mySubscript["\[Del]h",ToString[i]],{i,1,Length[h]}];
		dh=Grad[h,vars];
		vVars=Table[mySubscript[v,i],{i,1,Length[h]}];
		If[prSetup,
			Print["***** EQ constraints and their gradients"];
			PrintLabelledList[Join[Thread[h==0],dh],
			Join[hLabels,dhLabels],2]
		]
	];
	{L, eqns} = LagrangianEquations[{vars,uVars,sVars,vVars},f, g, h];
	If[prSetup,
		PrintLabelledList[{L,eqns},
		{"***** Lagrangian","\[Del]L=0"},1]
	];
	soln=AllKTpoints[{vars,uVars,sVars,vVars},
		f,g,dg,h,dh,active,prResults,L,eqns, opts];
	If[prSetup,
		If[soln!={},
			solnPts=Map[ToRules[{vars,uVars,sVars,vVars}==#]&,soln];
			fStar=Map[fLabel->#&,f/.solnPts];
			Print["***** Valid KT Point(s) *****"];
			Print[TableForm[{fStar,solnPts}]],
			Print["***** No Valid KT Points Found *****"]
		]
	];
	soln 
];

KTSolution[f_,con_,vars_List,opts___?OptionQ]:=KTSolution[f,{con},vars,opts];

KTSolution[f_,con_,vars_,opts___?OptionQ]:=KTSolution[f,{con},{vars},opts];

(*AllKTpoints tries to compute all KT points by assuming all possible combinations of inequality constrainst to be active. All regular KT points are returned in a list containing values in the order: designVars, uVars, sVars, and vVars.*)

AllKTpoints [{x_,u_,s_,v_},f_, g_List, dg_List, h_List, dh_List, activeCases_, prResults_,L_, eqns_, opts___] :=
Module[{nx=Length[x], ng=Length[g], nh=Length[h], i=0, j,
	allVars={x, u, s, v}, possibleCases=activeCases, ktPoints,
	rgulr, allValidPoints={}, xs, us, ss, vs, status, 
	caseEqns, knownValues},

	If[possibleCases==={},
		possibleCases = Select[BinarySubsets[Range[ng]],Length[#]<=nx-nh&]
	];
	While [i<Length[possibleCases],++i;
		active = possibleCases[[i]];
		{ktPoints,knownValues,caseEqns} = 
			SolveKTcase[allVars, eqns,active, opts];
		If[prResults,
			Print["************ Case "<>ToString[i]<>" ************"];
			Print["Active inequalities"->If[active=={},"None",active]];
			Print["Known values"->knownValues];
			PrintLabelledList[{caseEqns},{"Equations for this case"}];
			If[ktPoints==={},Print["No solution for this case"]]
		];
		j = 0;
		While [j<Length[ktPoints],++j;
			status = "Invalid KT Point";
			{xs, us, ss, vs}=ktPoints[[j]];
			rgulr = CheckRegularity[x,xs,dg, dh, active];
			If[And[rgulr[[3]]==="RegularPoint",
				FreeQ[Negative[us],True], FreeQ[Negative[ss], True]],
				status="Valid KT Point";
				AppendTo[allValidPoints,{xs, us, ss, vs}]
			];
			If[prResults,
				Print["-----Solution "<>ToString[j]<>"-----"];
 				PrintLabelledList[{xs},{MatrixForm[x]}];
 				If[g!={},
 					PrintLabelledList[{us,ss},
 					{MatrixForm[u],MatrixForm[s]}]
 				];
 				If[h!={},
 					PrintLabelledList[{vs},{MatrixForm[v]}]
 				];
				If[Length[h]+Length[active]>1,
					PrintLabelledList[rgulr,{
					"Constraint gradient matrix","Rank",
					"Regularity status"},2]
				];
				PrintLabelledList[{status,
					N[f/.ToRules[x==ktPoints[[j,1]]]]},
					{"KT Status","Objective function value"}]
			];
		]
	]; Union[allValidPoints, SameTest->(Norm[Flatten[#1]-Flatten[#2]]\[LessEqual]10^(-7)&)]
];

(*LagrangianEquations [vars_List, f, g_List, h_List] constructus Lagrangian, L = f + Sum[ui(gi + si^2) + vi hi]. The variables are assumed to be in the following order: designVars, uVars, sVars, and vVars where designVars = actual design variables, uVars = multipliers for inequality constraints, sVars = slack variables and vVars = multipliers for equality constraints. It	returns the Lagrangian function and 	a list of equations obtained by setting the gradient of the	Lagrangian to zero. The constraints can be omitted if not present.*)

LagrangianEquations[{x_, u_, s_, v_}, f_, g_List, h_List] :=
Module [{vars, L, ng = Length[g],
	nh = Length[h], i, eqns, ss},
	L = f + Apply[Plus,u(g+s)] + Apply[Plus,v h];
	ss = Map[First,s];
	eqns = Flatten[{Grad[L,x],g+s,h, 2 u ss}];
	eqns = Map[#==0&,eqns];
	{L, eqns}
];

(*SolveKTcase [vars, eqns, active], solve	a given case of KT conditions. The active list contains the numbers of inequality constraints that are assumed active.	The active list can be omitted if there are no active constraints. The values returned are in the following order: designVars, uVars, sVars, and vVars.*)

SolveKTcase[{x_, u_, s_, v_}, eqns_List, active_List, opts___] := 
Module [{i, soln={}, xVars=x, vVars=v, uVars=u,
	slackVars, inactive, zeroMultipliers,zeroSlacks, knownValues,
	caseEqns, method, strtSoln, unknowns},
	{method, strtSoln} = {SolveEquationsUsing,StartingSolution} /. 
		{opts} /. Options[KTSolution];
	inactive=Complement[Table[i,{i,1,Length[uVars]}],active];
	slackVars = Map[First,s];
	zeroMultipliers = Table[uVars[[inactive[[i]]]]->0,
		{i,1,Length[inactive]}];
	zeroSlacks = Table[slackVars[[active[[i]]]]->0,
		{i,1,Length[active]}];
	knownValues = Flatten[{zeroMultipliers,zeroSlacks}];
	caseEqns=eqns /. knownValues;
	caseEqns=Flatten[Map[If[FreeQ[#,True],#,{}]&,caseEqns]];
	If[FreeQ[caseEqns,False],
		If[method === FindRoot,
			unknowns = Complement[Flatten[{x, u, slackVars, v}]/.
				knownValues,{0}];
			If[MatrixQ[strtSoln],
				strtSoln = unknowns/.Table[x[[i]]->strtSoln[[i]],
				{i,1,Length[x]}];
				strtSoln = strtSoln /.
					Map[#->{1.,1.1}&,Flatten[{x, u, slackVars, v}]],
				strtSoln = unknowns/.Table[x[[i]]->strtSoln[[i]],
				{i,1,Length[x]}];
				strtSoln = strtSoln /.
					Map[#->1.&,Flatten[{x, u, slackVars, v}]]
			];
			soln = FindRoot[caseEqns,	
				Evaluate[Apply[Sequence,Transpose[{unknowns,strtSoln}]]],
				Evaluate[FilterOptions[FindRoot,opts]],Compiled->False];
			If[Head[soln]===method,soln={},soln={soln}],
			soln = method[caseEqns]
		];
	];
	If[Head[soln]===method,soln={}];
	If[soln!={},
		slackVars = slackVars /. zeroSlacks;
		uVars = uVars /. zeroMultipliers;
		soln = Union[Chop[{xVars, uVars, 
		slackVars^2, vVars} /. soln]];
		(*Remove complex solutions*)
		soln=Select[soln, FreeQ[#,_Complex,2]&]
	];{soln, knownValues, caseEqns} 
];

(*CheckRegularity [vars, point, dg, dh, activeG]	Checks if the given point is a regular point. dg and dh	contains gradients of inequality and equality constraints. activeG contains a list of active inequality constraints.*)

CheckRegularity [vars_List, point_List, dg_List, dh_List, activeG_List] :=
Module[{i, dgActive, a={}, b, rnk=1,
	na=Length[activeG]+Length[dh],
	status = "IrregularPoint"},
	dgActive=Table[dg[[activeG[[i]]]],
		{i,1,Length[activeG]}];
	b = Join[dgActive, dh]/.ToRules[vars == point];
	If [Length[b] >= 2, 
		a = Transpose[b];
		rnk = Rank[a];
		If [rnk==na,status = "RegularPoint"],
		a = b;
		status = "RegularPoint"
	];
	{a, rnk,status}
];

NewFUsingSensitivity[vars_, f_, sol_, g_List, modg_List] :=
Module [{n=Length[vars], x, u, fs, sc, mc, db, newf},
	{x,u} = {Take[sol,n],Take[sol,{n+1,n+Length[g]}]};
	sc = Map[getConstantTerm,g];
	mc = Map[getConstantTerm,modg];
	db = -(mc - sc);
	fs = f /. ToRules[vars == x];
	newf = fs - Apply[Plus,u*db];
	PrintLabelledList[{g,modg,db},{"Original constraints",
	"Modified constraints","\[CapitalDelta]b"},1];
	PrintLabelledList[{fs,newf},{"Original f","New f"},2];
];
	getConstantTerm[a_]:=Module[{m={},exp=a},
		If[Or[Head[a]===LessEqual,Head[a]===Equal],
			exp = Apply[List,a];
			exp = exp[[1]] - exp[[2]]
		];
		If[Head[a]===GreaterEqual,
			exp = Apply[List,a];
			exp = -exp[[1]] + exp[[2]]
		];
		If[Length[exp]>1,
			m = Select[Apply[List,Expand[exp]],NumericQ]];
		If[m==={},0,m[[1]]]
	];

ConvertToStandardNLPForm[a_List]:=Map[ConvertToStandardNLPForm,a];
ConvertToStandardNLPForm[a_]:=Module[{exp=a},
	exp = Apply[List,a];
	exp = exp[[1]] - exp[[2]];
	Switch[a,
		_LessEqual, exp=exp\[LessEqual]0,
		_Equal, exp=exp==0,
		_GreaterEqual,exp=-exp\[LessEqual]0
	];exp
];



ConstrainedSufficiency[f_, g_, vars_, sol_, opts___?OptionQ] := 
    Module[{n=Length[vars],
    fd,gg=Flatten[{g}], hf, hg, x, u, xv, dg, HL, 
    dv, dd, prLevel, prResults, dHd},
    {prLevel,dd}={PrintLevel,FeasibleChangesName}/.
    	Flatten[{opts, Options[ConstrainedSufficiency]}];
    If[prLevel>0,prResults=True,prResults=False];
	{x,u} = {Take[sol,n],Take[sol,{n+1,n+Length[gg]}]};
	xv = ToRules[vars == x];
	hf = Hessian[f, vars] /. xv;
	dv = Table[mySubscript[dd,i], {i, 1, n}];
	fd = dv;
	If[Length[gg]\[GreaterEqual]1,
	 	gg = Map[First[#]&, ConvertToStandardNLPForm[gg]]; 
	 	dg = Grad[gg, vars] /. xv;
     	eqns = Thread[dg.dv == 0];
     	(*fd = dv /. First[Solve[eqns, dv[[Range[Length[gg]]]]]];*)
     	fd = dv /. First[Solve[eqns]];
	 	hg = Hessian[gg, vars] /. xv;      
     	HL = hf + u.hg;
		If[prResults,
	      	PrintLabelledList[hg,Table[mySubscript["Hg", i],
      		{i, 1, Length[gg]}], 2];
	      	PrintLabelledList[{dg, eqns, fd},
      		{"Gradients", "Equations", "Feasible changes"}, 1]
		], 
      	HL = hf
     ];dHd=Expand[fd.HL.fd];
     If[prResults,
      	PrintLabelledList[{hf, HL, Chop[dHd]}, {"Hf","HL", "d.HL.d"}, 2];
      ]; 
      dHd
]; 



ConstructDual[f_, g_, vars_,opts___?OptionQ] := 
    Module[{gg,u={},v={}, uu, uue, cons, L=f,le=0,eq=0,prResults,
    prLevel, dvars},
    {prLevel,{uu,vv}}={PrintLevel,DualVariableNames}/.
    	Flatten[{opts,Options[ConstructDual]}];
    If[prLevel>0,prResults=True,prResults=False];
	gg = ConvertToStandardNLPForm[Flatten[{g}]];
	Do[	Switch[Head[gg[[i]]],
			LessEqual,le=le+1;uue=mySubscript[uu,le];L=L+uue First[gg[[i]]];
			u=Flatten[{u,uue}], 

			Equal,eq=eq+1;uue=mySubscript[vv,eq];L=L+uue First[gg[[i]]];
			v=Flatten[{v,uue}]
		],{i,1,Length[gg]}
	];
	cons = Flatten[{Map[#==0&,Grad[L,vars]],Map[#\[GreaterEqual]0&,u]}];
	dvars = Flatten[{vars,u,v}];
	If[prResults,
		PrintLabelledList[{{dvars},L,cons}, 
		{"Dual problem: Variables","Maximize","Subject to"}, 1];
	]; 
	{L,cons,dvars}
]; 



End[];

EndPackage[]