(***********************************************************************
This file was generated automatically by the Mathematica front end.
It contains Initialization cells from a Notebook file, which typically
will have the same name as this file except ending in ".nb" instead of
".m".

This file is intended to be loaded into the Mathematica kernel using
the package loading commands Get or Needs.  Doing so is equivalent to
using the Evaluate Initialization Cells menu command in the front end.

DO NOT EDIT THIS FILE.  This entire file is regenerated automatically 
each time the parent Notebook file is saved in the Mathematica front end.
Any changes you make to this file will be overwritten.
***********************************************************************)









BeginPackage["OptimizationToolbox`QuadraticProgramming`",
"OptimizationToolbox`CommonFunctions`"];

MakeQuadraticForm::usage="MakeQuadraticForm[f,vars], given f returns vector c and coefficient matrix Q for the quadratic form f = cT.x + 1/2 xT.Q.x.";

QP::"NonConvex" = "QP is non-convex. Principle minors of matrix Q are `1`.";

ActiveSetQP::"Phase-I" = "Trying PrimalAffine interior point method. LinearProgramming failed to find an initial feasible solution.";

ActiveSetQP::"Infeasible" = "Failed to find a starting feasible solution.";

ActiveSetDualQP::"CGDir" = "Conjugate gradient direction is not a descent direction at `1`.";



FormDualQP::usage = "FormDualQP[f, g, vars, options]. Forms dual of the given QP. f is the objective function, g is a list of constraints, and vars is a list of variables. See Options[FormDualQP] to find out about a list of valid options for this function.";

Options[FormDualQP] =  
	{UnrestrictedVariables->{},
	ProblemType->Min, StandardVariableName->"x",
	DualQPVariableNames->{"u","v"}};

DualQPVariableNames::usage="DualQPVariableNames->symbols to use when creating dual variable names. The default is {u, v}.";



PrimalAffineQP::usage = "PrimalAffineQP[f, g, vars, options]. Solves a convex QP problem using the primal affine algorithm. f is the objective function, g is a list of constraints, and vars is a list of variables. See Options[PrimalAffineQP] to find out about a list of valid options for this function.";

Options[PrimalAffineQP] =  
	{UnrestrictedVariables->{}, MaxIterations->20,
	ProblemType->Min, StandardVariableName->"x",
	PrintLevel->1,
	StepLengthFactor->0.99,
	ConvergenceTolerance->{0.001,0.2,2,0.5},
	StartingVector->{}};

StepLengthFactor::usage="StepLengthFactor is an option for interior point methods. It is the reduction factor applied to the computed step length to maintain feasibility. The default is 0.99";

StartingVector::usage="StartingVector is an option for several interior point methods. The default is {1,\[Ellipsis],1}.";



PrimalDualQP::usage = "PrimalDualQP[f, g, vars, options]. Solves a QP problem using the interior point algorithm based on solving KT conditions using the Newton-Raphson method. f is the objective function, g is a list of constraints, and vars is a list of variables. See Options[PrimalDualQP] to find out about a list of valid options for this function.";

Options[PrimalDualQP] =  
	{UnrestrictedVariables->{}, MaxIterations->20,
	ProblemType->Min, StandardVariableName->"x",
	PrintLevel->1,
	StepLengthFactor->0.99,
	ConvergenceTolerance->0.0001,
	StartingVector->{}};

PrimalDualQPSolver::usage = "PrimalDualQPSolver[c0, c, Q, A, b, startVec, maxIter, \[Beta], tol, prResults]. Using primal-dual interior point method, it solves a QP problem expressed in matrix form, Minimize c0 + c.x + 1/2 x.Q.x subject to A.x==b and x\[GreaterEqual]0. startVec is the starting solution vector (use {} for automatic generation of startVec), maxIter = maximum number of iterations allowed, \[Beta] = step length factor, tol = convergence tolerance, and prResults = True/False controls printing of intermediate results.";



ActiveSetQP::usage = "ActiveSetQP[f, g, vars, options]. Solves a QP problem using the active set algorithm. f is the objective function, g is a list of constraints, and vars is a list of variables. See Options[ActiveSetQP] to find out about a list of valid options for this function.";

Options[ActiveSetQP] =  
	{MaxIterations->20,
	ProblemType->Min,
	PrintLevel->1,
	ConvergenceTolerance->0.0001};

ActiveSetQPSolver::usage = "ActiveSetQPSolver[Q, c, aEQ, bEQ, aLE, bLE, maxIter, tol, prResults]. Lower level ActiveSetQPSolver. The objective function = c.vars + 1/2 vars . Q. vars. aEQ, bEQ, aLE, and bLE are constraint matrices and their rhs (EQ for = and LE for \[LessEqual]). maxIter = maximum number of iterations. tol = convergence tolerance, and prResults = True/False for printing of results.";



ActiveSetDualQP::usage = "ActiveSetDualQP[f, vars, opts]. Computes minimum of a quadratic function subject to non-negativity constraints using Conjugate Gradient method. User has the option of using either 'PolakRibiere' (default) or 'FletcherReeves' method. See Options[ActiveSetDualQP] to see a list of options for the function. The function returns {x, Status}. x is either the optimum point or the next point after MaxIterations. Status indicates whether the point returned is optimum or not.";

Options[ActiveSetDualQP] = {
	ProblemType->Min, StartingVector->{},
	UnrestrictedVariables->{},
	PrintLevel->1,
	MaxIterations->10,
	ConvergenceTolerance->0.001,
	Method->"PolakRibiere"};

Method::usage="Method->Method used for conjugate direction finding. The choice is between PolakRibiere (default) and FletcherReeves.";

ActiveSetDualQPSolver::usage = "ActiveSetDualQPSolver[c0,c, Q, nonNeg, maxIter, eps, method, strtVec, vars, prResults]. Lower level ActiveSetDualQPSolver. The objective function = c0 + c.vars + 1/2 vars . Q. vars. nonNeg contains a list of variables restricted to be positive. maxIter = maximum number of iterations. eps = convergence tolerance, method = PolakRibiere (default) or FletcherReeves method, strtVec is the starting vector, vars = list of variables, and prResults = True/False for printing of results.";



Begin["`Private`"];

Needs["Utilities`FilterOptions`"];
Needs["LinearAlgebra`MatrixManipulation`"];





MakeQuadraticForm[f_,vars_]:=Module[{c0,c,Q},
	c0 = getConstant[f,vars];
	c = Coefficient[f,vars]/.Map[#->0&,vars];
	Q = Map[Coefficient[f,#*vars]&,vars];
	Do[Q[[i,i]]=2*Q[[i,i]],{i,1,Length[vars]}];
	{c0, c, Q}
];

ConvertToStandardLP[type_, f_, g_, designVars_, unrestrictedVars_, 
varName_, prResults_:True] :=
Module[{n, nf = f, ng = g, h={}, con, vars},
	n=Length[designVars]+Length[unrestrictedVars];
	vars = Table[mySubscript[varName,i], {i,1,n}];
	If[type===Max, nf = -f];
	nf = ConvertUnrestricted[nf, designVars, unrestrictedVars, vars];
	ng = Map[StandardizeConstraint[#,designVars]&,g];
	ng = Map[ConvertUnrestricted[#, designVars, unrestrictedVars, vars]&,ng];
	Do[	con=ng[[i]];
		Switch[Head[con],
			LessEqual, newVar=mySubscript[varName,Length[vars]+1];
				AppendTo[h,First[con] + newVar==Last[con]];
				AppendTo[vars,newVar],
			GreaterEqual, newVar=mySubscript[varName,Length[vars]+1];
				AppendTo[h,First[con] - newVar==Last[con]];
				AppendTo[vars,newVar],
			Equal,
				AppendTo[h,con]
		],{i,1,Length[ng]}
	];
	If[prResults, PrintLabelledList[{nf, h, {Thread[vars\[GreaterEqual]0]}},
		{"Minimize","   Subject to","   and"},1]
	];
	{nf, h, vars}
]       

ConvertUnrestricted[fcn_, vars_, unrestrictedVars_, x_] := 
Module[{xv={},p=1,i},
	Do[If[MemberQ[unrestrictedVars,i],
			AppendTo[xv,{vars[[i]]->x[[p]]-x[[p+1]]}];
			p=p+2,
			AppendTo[xv,{vars[[i]]->x[[p]]}];
			p=p+1
		],{i,1,Length[vars]}
	];		
	Map[Expand,fcn /. Flatten[xv]]
]

getConstant[poly_, vars_] := Module[{c},
	If[NumericQ[poly], poly,
		c=Map[Fold[Coefficient[#1, #2, 0]&,#,vars]&, poly];
		If[c===poly,0,c]
	]
]; 

getCoefficients[poly_, vars_] := Map[Coefficient[poly,#]&, vars]; 

StandardizeConstraint[g_, vars_] :=
Module[{a,b, ng},
	Switch[Head[g],
		LessEqual, poly = g[[1]] - g[[2]];
			a = getCoefficients[poly,vars];
			b = getConstant[poly, vars];
			If[b\[LessEqual]0, ng = Expand[a.vars] \[LessEqual] -b, 
				ng = Expand[-a.vars] \[GreaterEqual] b
			],
		GreaterEqual, poly = g[[1]] - g[[2]];
			a = getCoefficients[poly,vars];
			b = getConstant[poly, vars];
			If[b\[LessEqual]0, ng = Expand[a.vars] \[GreaterEqual] -b,
				ng = Expand[-a.vars] \[LessEqual] b
			],
		Equal, poly = g[[1]] - g[[2]];
			a = getCoefficients[poly,vars];
			b = getConstant[poly, vars];
			If[b\[LessEqual]0, ng = Expand[a.vars] == -b,
				ng = Expand[-a.vars] == b
			]
	];
	ng
]



FormDualQP[f_, g_, designVars_,opts___?OptionQ]:=
Module[{n,freeIndices,unrestrictedVars, type, varName, u,v,du,dv,
	vars, sf, sg, c0, c, A, b, Q,x, df, QI, rhs, cons, dvars},
	{unrestrictedVars, type, varName, {u,v}}= 
		{UnrestrictedVariables, ProblemType,
		StandardVariableName, DualQPVariableNames} /. {opts} /. 
		Options[FormDualQP];
	freeIndices = Sort[Flatten[Map[
		Position[designVars,#]&,unrestrictedVars]]];
	StylePrint["Primal problem",FontWeight->"Bold"];
	{sf, sg,vars} = ConvertToStandardLP[type, f, Flatten[g],
		designVars,freeIndices, varName];
	n=Length[vars];
	{c0,c,Q}=MakeQuadraticForm[sf,vars];
	b=Map[Last[#]&,sg];
	A=Map[Coefficient[First[#],vars]&,sg];
	PrintLabelledList[{A,b,c,Q},{"A","b","c","Q"},2];
	du = Table[mySubscript[u,i],{i,1,Length[vars]}];
	dv = Table[mySubscript[v,i],{i,1,Length[b]}];
	rhs = Expand[-c+du+Transpose[A].dv];
	If[Rank[Q]===Length[Q],
		dvars = Flatten[{du,dv}];
		cons = Thread[du\[GreaterEqual]0];
		QI= Inverse[Q];
		x = Expand[QI.rhs];
		PrintLabelledList[{QI, rhs},{Superscript["Q",-1],"-c+u+AT.v"},2];
		PrintLabelledList[{Thread[vars->x]},{"Solution"}],
		x = vars;
		dvars = Flatten[{du,dv,x}];
		cons = Flatten[{Thread[Expand[Q.x - rhs] == 0],Thread[du\[GreaterEqual]0]}]
	];
	df = Expand[-1/2 x.Q.x + dv.b];
	StylePrint["Dual QP problem",FontWeight->"Bold"];
	PrintLabelledList[{{dvars},df,cons},
	{"Variables","Maximize","Subject to "},1];
	{df,cons,dvars,dv}
];



PrimalAffineQP[f_, g_, designVars_,opts___?OptionQ]:=
Module[{n,freeIndices,unrestrictedVars, maxIter, type, varName,
	prLevel, prSetup, prResults, \[Beta],tol,vars, sf,
	sg, sol={}, fstar={}, varsMapping, k, i, convgParam,
	c0, c, A, A1,b, iter=0, status="NonOptimum",
	x,c1,x1,phase,history},
	{unrestrictedVars, maxIter, type, varName, prLevel,
		\[Beta],tol,xk} = 
		{UnrestrictedVariables, MaxIterations, ProblemType,
		StandardVariableName, PrintLevel,
		StepLengthFactor,ConvergenceTolerance,
		StartingVector} /. Flatten[{opts, Options[PrimalAffineQP]}];
	Switch[prLevel,
		0, {prSetup, prResults}={False, False}, 
		1, {prSetup, prResults}={True, False}, 
		_, {prSetup, prResults}={True, True}
	];
	freeIndices = Sort[Flatten[Map[
		Position[designVars,#]&,unrestrictedVars]]];
	{sf, sg,vars} = ConvertToStandardLP[type, f, Flatten[g],
		designVars,freeIndices, varName, prSetup];
	varsMapping = {};
	k = 1;
	Do[If[FreeQ[freeIndices,i],
		varsMapping={varsMapping,designVars[[i]]->vars[[k]]};
		k = k + 1,
		varsMapping={varsMapping,
			designVars[[i]]->vars[[k]]-vars[[k+1]]};
		k = k + 2],
		{i,1,Length[designVars]}
	];
	varsMapping=Flatten[varsMapping];
    If[prSetup, Print["Problem variables redefined as:",varsMapping]];
	n=Length[vars];
	If[xk==={},xk=Table[1,{n}]];
	{c0,c,Q}=MakeQuadraticForm[sf,vars];
	b=Map[Last[#]&,sg];
	A=Map[Coefficient[First[#],vars]&,sg];
	If[A==={},A={Table[0,{n}]};b={0}];
	If[prSetup,
		PrintLabelledList[{A,b,c,Q},{"A","b","c","Q"},2]
	];
	pm = PrincipalMinors[Q];
	If[Min[pm]\[GreaterEqual]0,
		If[prSetup,
			Print["Convex problem. Principal minors of Q"->pm]
		],
		Message[QP::"NonConvex", pm]
	];
	x1=Join[xk,{1}];
	history = {xk};
	If[prResults,
		Print["Starting point"->xk,
			" Objective function"->sf/.ToRules[vars==xk],
			" Status"->status]
	];
	(* Phase I *)
	If[Norm[A.xk-b]/(Norm[b]+1)>tol[[2]],
		While[And[status==="NonOptimum",iter<maxIter],
			++iter;
			phase = 1;
			If[prResults,
				StylePrint["***** Iteration "<>ToString[iter]<>
				" (Phase "<>ToString[phase]<>") *****",
				"Output", FontWeight->"Bold"];
			];
			A1=Transpose[Join[Transpose[A],{b-A.xk}]];
			c1=Join[Table[0,{n}],{Last[x1]}];
			{x1,status,convgParam}=PrimalAffineLPStep[
				n+1,A1,b,c1,x1,\[Beta],tol,prResults];
			xk=Drop[x1,-1];
			AppendTo[history,xk];
			If[prResults,
				Print["New point"->xk,
				" Artificial objective function"->Last[x1],
				" Objective function"->sf/.ToRules[vars==xk],
				" Status"->status]
			]
		]
	];
	(* Phase II *)
	If[And[iter<maxIter,Norm[b-A.xk]/(Norm[b]+1)>tol[[2]]],
		Print["Phase I ended with no feasible solution"];
		Print["New point"->xk];
		Print["Convergence parameters"->convgParam];
		Print["Constraint values norm"->Norm[b-A.xk]/(Norm[b]+1)];
		Abort[];
	];
	status = "NonOptimum";
	While[And[status==="NonOptimum",iter<maxIter],
		++iter;
		phase = 2;
		If[prResults,
			StylePrint["***** Iteration "<>ToString[iter]<>
			" (Phase "<>ToString[phase]<>") *****",
			"Output", FontWeight->"Bold"];
		];
		{xk,status,convgParam}=PrimalAffineQPStep[
			n,A,b,c,Q,xk,\[Beta],tol,prResults];
		AppendTo[history,xk];
		If[prResults,
			Print["New point"->xk,
			" Objective function"->sf/.ToRules[vars==xk],
			" Status"->status]
		];
	];
	sol = varsMapping /. ToRules[vars==xk];
	fstar = f/.sol;
	If[prSetup,
		StylePrint["***** "<>ToString[status]<>" solution after "<>
		ToString[iter]<>" iterations *****",
		"Output",FontWeight->"Bold"];
		Print["Interior solution"->sol," Objective function"->fstar];
		Print["Convergence parameters"->convgParam]
	];
	{{fstar,sol},history}
];

PrimalAffineQPStep[n_,A_,b_,c_,Q_,x_,\[Beta]_,\[Epsilon]_, prResults_]:=
	Module[{newx=x,e=Table[1,{n}],H,AH,Qxc,w,AT=Transpose[A],s,
			status="NonOptimum",r,\[Alpha]=0,\[Alpha]2=$MaxNumber,
			dQd, Tk2, H1, lhs, rhs, \[Sigma]1,\[Sigma]2,\[Sigma]3,\[Sigma]4},
		Tk2 = 1/x^2;
		H1 = Q+DiagonalMatrix[Tk2]; 
		H=Inverse[H1];
		AH=A.H;
		Qxc=Q.x+c;
		{lhs, rhs} = {AH.AT,AH.Qxc};
		w = LinearSolve[lhs, rhs];
		s = Qxc - AT.w;
		d = -H.s;
		\[Sigma]1 = Norm[d];
		\[Sigma]2 = Norm[A.x-b]/(Norm[b]+1);
		\[Sigma]3 = Norm[s]/(Norm[Qxc]+1);
		\[Sigma]4 = Abs[x.s];
		If[prResults,
			Print["Tk2[diagonal]"->Tk2];
			PrintLabelledList[{H1, H, AH, lhs},
			{"Q+Tk2","H","A.H","A.H.AT"},1];
			PrintLabelledList[{Qxc,rhs},{"Qx+c","A.H.(Qx+c)"},2];
			PrintLabelledList[{w,s,d},{"w","s","d"},3];
			Print["Convergence parameters"->{\[Sigma]1,\[Sigma]2,\[Sigma]3,\[Sigma]4}]
		];
		If[Apply[And,Flatten[{Thread[{\[Sigma]1,\[Sigma]2,\[Sigma]3,\[Sigma]4}\[LessEqual]\[Epsilon]],Min[x]\[GreaterEqual]0.}]],
			status="Optimum",
			dQd = d.Q.d;
			If[dQd>0,\[Alpha]2=-(d.Qxc)/dQd];
			If[Min[d]\[GreaterEqual]0,\[Alpha]=\[Alpha]2,
				\[Alpha]=Min[\[Alpha]2, \[Beta] Min[Select[-x/d,Positive]]]
			];
			newx=x+\[Alpha] d;
			If[prResults,
				PrintLabelledList[{d.Qxc, dQd, \[Alpha]2},
					{"d.(Qx+c)", "d.Q.d", "\[Alpha]2"},3];
				PrintLabelledList[{{-\[Beta] x/d},\[Alpha]},{"-\[Beta]x/d","\[Alpha]"},1]
			];
		];
		{newx,status,{\[Sigma]1,\[Sigma]2,\[Sigma]3,\[Sigma]4}}
	];

PrimalAffineLPStep[n_,A_,b_,c_,x_,\[Beta]_,\[Epsilon]_, prResults_]:=
Module[{newx=x,Tk=DiagonalMatrix[x],w,ATk,AT=Transpose[A],
	status="NonOptimum",r,\[Alpha]=0,lhs, rhs,d,\[Sigma]1,\[Sigma]2,\[Sigma]3,\[Sigma]4},
	ATk=A.(Tk^2);
	lhs = ATk.AT; rhs = ATk.c;
	w=LinearSolve[lhs, rhs];
	r=c-AT.w;
	d=-Tk.r;
	\[Sigma]1=Norm[d];
	\[Sigma]2=Norm[(A.x)-b]/(Norm[b]+1);
	\[Sigma]3=Norm[r]/(Norm[c]+1);
	\[Sigma]4=Abs[c.x - b.w];
	If[Apply[And,Flatten[{Thread[{\[Sigma]1,\[Sigma]2,\[Sigma]3,\[Sigma]4}\[LessEqual]\[Epsilon]],Min[x]\[GreaterEqual]0.}]],
	 status="Optimum",
		If[Apply[And,Map[NonNegative,Chop[d]]],status="Unbounded",
			\[Alpha] = \[Beta] Min[Select[-1/d,Positive]];
			newx=x+\[Alpha] Tk.d
		]
	];
	If[prResults,
		Print["Tk[diagonal]"->x];
		PrintLabelledList[{lhs},{"A.Tk.Tk.AT"},1];
		PrintLabelledList[{rhs, w, r, d},
		{"A.Tk.Tk.c","w","r","d"},2];
		Print["Convergence parameters"->{\[Sigma]1,\[Sigma]2,\[Sigma]3,\[Sigma]4},
			" \[Beta](-1/d)"->-\[Beta]/d," Step length, \[Alpha]"->\[Alpha]]
	];
	{newx,status,{\[Sigma]1,\[Sigma]2,\[Sigma]3,\[Sigma]4}}
];



PrimalDualQP[f_, g_, designVars_,opts___Rule]:=
Module[{m,n,freeIndices,unrestrictedVars, maxIter, type, varName,
	prLevel, prSetup, prResults, \[Beta], tol, vars, sf,
	sg, sol={}, fstar={}, startVec, varsMapping, k, i,
	Q, c0, c, A, b, iter=0, status="NonOptimum", xk, uk, vk, history},
	{unrestrictedVars, maxIter, type, varName, prLevel,
		\[Beta],tol,startVec} = 
		{UnrestrictedVariables, MaxIterations, ProblemType,
		StandardVariableName, PrintLevel,
		StepLengthFactor,ConvergenceTolerance,
		StartingVector} /. {opts} /. Options[PrimalDualQP];
	Switch[prLevel,
		0, {prSetup, prResults}={False, False}, 
		1, {prSetup, prResults}={True, False}, 
		_, {prSetup, prResults}={True, True}
	];
	freeIndices = Sort[Flatten[Map[
		Position[designVars,#]&,unrestrictedVars]]];
	{sf, sg,vars} = ConvertToStandardLP[type, f, Flatten[g],
		designVars,freeIndices, varName, prSetup];
	varsMapping = {};
	k = 1;
	Do[If[FreeQ[freeIndices,i],
		varsMapping={varsMapping,designVars[[i]]->vars[[k]]};
		k = k + 1,
		varsMapping={varsMapping,
			designVars[[i]]->vars[[k]]-vars[[k+1]]};
		k = k + 2],
		{i,1,Length[designVars]}
	];
	varsMapping=Flatten[varsMapping];
    If[prSetup,Print["Problem variables redefined as:",varsMapping]];
	{c0,c,Q}=MakeQuadraticForm[sf,vars];
	b=Map[Last[#]&,sg];
	A=Map[Coefficient[First[#],vars]&,sg];
	n=Length[vars];
	If[A==={},A={Table[0,{n}]};b={0}];
	If[prSetup,
		PrintLabelledList[{A,b,c,Q},{"A","b","c","Q"},2]
	];
	pm = PrincipalMinors[Q];
	If[Min[pm]\[GreaterEqual]0,
		If[prSetup,
			Print["Convex problem. Principal minors of Q"->pm]
		],
		Message[QP::"NonConvex", pm]
	];
	{xk, uk, vk, status, history} = PrimalDualQPSolver[c0, c, Q, A, b,
		startVec, maxIter, \[Beta], tol, prResults];
	sol = varsMapping /. ToRules[vars==xk];
	fstar = f/.sol;
	iter = Length[history]-1;
	If[prSetup,
		StylePrint["***** "<>ToString[status]<>" solution after "<>
		ToString[iter]<>" iterations *****","Output",FontWeight->"Bold"];
		Print["Interior solution"->sol];
		Print["Objective function"->fstar]
	];
	{{fstar,sol}, history}
];

PrimalDualQPSolver[c0_, c_, Q_, A_, b_, startVec_, maxIter_, \[Beta]_, tol_, prResults_]:=
Module[{m, n, xk, uk, vk, iter = 0, status = "NonOptimum", history={}},
	m=Length[b];
	n=Length[c];
	If[startVec==={},xk=Table[1.0,{n}];uk=xk;vk=Table[0.0,{m}],
		{xk,uk,vk}=startVec];
	If[prResults,
		Print["**** Starting vectors ****"];
		Print["Primal vars (x)"->xk];
		Print["Dual vars (u)"->uk];
		Print["Multipliers (v)"->vk];
		PrintLabelledList[{c0+c.xk+1/2 xk.Q.xk,status},
			{"Objective function","Status"}]
	];
	AppendTo[history,xk];
	While[And[status==="NonOptimum",iter<maxIter],
		++iter;
		If[prResults,
			StylePrint["***** Iteration "<>ToString[iter]<>" *****",
			"Output",FontWeight->"Bold"];
		];
		{xk,uk,vk,status}=PrimalDualQPStep[
			iter,Q,A,b,c,xk,uk,vk,\[Beta],tol,prResults];
		AppendTo[history,xk];
		If[prResults,
			PrintLabelledList[{c0+c.xk+1/2 xk.Q.xk, status},
			{"Objective function","Status"}]
		];
	];
	{xk, uk, vk, status, history}
];

PrimalDualQPStep[k_,Q_,A_,b_,c_,xk_,uk_,vk_,\[Beta]_,\[Epsilon]_, prResults_]:=
Module[{n=Length[xk],newx=xk,newu=uk, newv=vk,X=DiagonalMatrix[xk],
	U=DiagonalMatrix[uk],e,
	AT=Transpose[A],XQU,XQU1,AXQU,XAT,xrd,lhs, rhs,
	status="NonOptimum",rp,rd,rc,\[Alpha]p=0,\[Alpha]d=0,dxk, 
	duk,\[Sigma]p,\[Sigma]d,\[Mu],dv=0,du=0,dx=0},
	e = Table[1,{n}];
	rp=-A.xk+b;
	rd=Q.xk-AT.vk-uk+c;
	\[Mu] = ((xk.uk)/n)/k;
	rc=-xk uk + \[Mu];
	\[Sigma]p=Norm[A.xk-b]/(Norm[b]+1);
	\[Sigma]d=Norm[rd]/(Norm[Q.xk+c]+1);
	If[prResults,
		PrintLabelledList[{rp, rd, rc},{mySubscript["r","p"],
			mySubscript["r","d"],mySubscript["r","c"]},3];
		PrintLabelledList[{\[Sigma]p, \[Sigma]d, \[Mu]},
			{"Parameters: \[Sigma]p", "\[Sigma]d", "\[Mu]"}]
	];
	If[Max[\[Sigma]p,\[Sigma]d,\[Mu]]\[LessEqual]\[Epsilon], status="Optimum",
		XQU1=X.Q + U;
		XQU=Inverse[X.Q + U];
		AXQU=A.XQU;
		XAT = X.AT;
		xrd = X.rd;
		lhs = AXQU.XAT;
		rhs = rp+AXQU.(xrd-rc);
		dv=LinearSolve[lhs, rhs];
		dx=XQU.(XAT.dv-xrd+rc);
		du=rc/xk - (uk/xk)dx;
		dxk = Table[If[dx[[i]]<0,-xk[[i]]/dx[[i]],\[Infinity]],{i,1,Length[dx]}];
		duk = Table[If[du[[i]]<0,-uk[[i]]/du[[i]],\[Infinity]],{i,1,Length[du]}];
		\[Alpha]p = Min[1, \[Beta] dxk];
		\[Alpha]d = Min[1, \[Beta] duk];
		newx=xk+\[Alpha]p dx;
		newu=uk+\[Alpha]d du;
		newv=vk+\[Alpha]d dv;
		If[prResults,
			PrintLabelledList[{xk, uk},
				{"X[diagonal]","U[diagonal]"},2];
			PrintLabelledList[{XQU1, XQU},
				{"X.Q + U","Inverse[X.Q + U]"},1];
			PrintLabelledList[{lhs},
				{"A.Inverse(X.Q + U).X.AT"},1];
			PrintLabelledList[{rhs, dv},
			{"rp+A.Inverse[X.Q + U](X.rd-rc)", "dv"},2];
			PrintLabelledList[{dx, du},{"dx","du"},2];
			PrintLabelledList[{{dxk},\[Alpha]p,{duk},\[Alpha]d},
			{"-x/dx",mySubscript["\[Alpha]","p"], "-u/du",mySubscript["\[Alpha]","d"]},2];
			PrintLabelledList[{{newx},{newu},{newv}},
			{"New primal vars","New dual vars", "New multipliers"},1];
		];
	];
	{newx,newu,newv,status}
];



ActiveSetQP[f_, g_, vars_,opts___Rule]:=
Module[{maxIter, sf, type,status,
	prLevel, prSetup, prResults, tol, sg, sol={}, fstar={}, hist,
	Q, c0, c, eqCons, leCons, geCons,
	aLE={},bLE={},aEQ={},bEQ={}, iter, xk, uk, vk},
	{maxIter, type, prLevel,tol} = 
		{MaxIterations, ProblemType,
		PrintLevel,
		ConvergenceTolerance} /. {opts} /. Options[ActiveSetQP];
	Switch[prLevel,
		0, {prSetup, prResults}={False, False}, 
		1, {prSetup, prResults}={True, False}, 
		_, {prSetup, prResults}={True, True}
	];
	If[type===Max,sf=-f,sf=f];
	{c0,c,Q}=MakeQuadraticForm[sf,vars];
	sg=Map[StandardizeConstraint[#,vars]&,Flatten[g]];
	eqCons=Select[sg,Head[#]===Equal&];
	leCons=Select[sg,Head[#]===LessEqual&];
	geCons=Select[sg,Head[#]===GreaterEqual&];
	If[leCons!={},
		bLE=Map[Last[#]&,leCons];
		aLE=Map[Coefficient[First[#],vars]&,leCons]
	];
	If[geCons!={},
		bLE=Join[bLE,-Map[Last[#]&,geCons]];
		aLE=Join[aLE,-Map[Coefficient[First[#],vars]&,geCons]]
	];
	If[eqCons!={},
		bEQ=Map[Last[#]&,eqCons];
		aEQ=Map[Coefficient[First[#],vars]&,eqCons]
	];
	If[prSetup,
		PrintLabelledList[{sf},{"Minimize"}];
		If[Length[bLE]>0,PrintLabelledList[{Thread[aLE.vars\[LessEqual]bLE]},
			{"LE Constraints"}]
		];
		If[eqCons!={},
			PrintLabelledList[{eqCons},{"EQ Constraints"}]
		];
		PrintLabelledList[{c,Q,aLE,bLE,aEQ,bEQ},
		{"c","Q","LE constraints: A","b",
			"EQ constraints: A","b"},2]
	];
	pm = PrincipalMinors[Q];
	If[Min[pm]\[GreaterEqual]0,
		If[prSetup,
			Print["Convex problem. Principal minors of Q"->pm]
		],
		Message[QP::"NonConvex", pm]
	];
	{{xk,uk,vk},status,hist,iter}=ActiveSetQPSolver[Q,c,aEQ,bEQ,aLE,bLE,
		maxIter,tol, prResults];
	sol = ToRules[vars==xk];
	fstar = f/.sol;
	If[aLE!={},
		If[Apply[Or,Thread[aLE.xk>bLE]],
			Print["Warning: Unable to satisfy all inequality constraints"];
			Print["The problem may be ill-conditioned. Solution after "<>
			ToString[iter]<>" iterations."];
			status="Non-Optimum";
			PrintLabelledList[{{sol}, {uk},{vk}, fstar},
			{"Solution","Lagrange multipliers for LE constraints (u)",
			"Lagrange multipliers for EQ constraints (v)",
			"Objective function"},1]
		]
	];
	If[prSetup,
		StylePrint["***** "<>ToString[status]<>" solution after "<>
		ToString[iter]<>" iterations *****","Output",FontWeight->"Bold"];
		PrintLabelledList[{{sol}, {uk},{vk}, fstar},
		{"Solution","Lagrange multipliers for LE constraints (u)",
		"Lagrange multipliers for EQ constraints (v)",
		"Objective function"},1]
	];
	{{fstar,sol}, hist}
];

ActiveSetQPSolver[Q_,c_,aEQ_,bEQ_,aLE_,bLE_,maxIter_,tol_, prResults_]:=
Module[{n=Length[c],xk,uk,vk,dk,gk,wk={},nwk,nh=Length[aEQ],ng=Length[aLE],
	A,b,lhs,rhs,sol,\[Epsilon]=10^(-10),status="NonOptimum",\[Alpha],iter=0,
	hist={},leCons={},eqCons={},uAll},

	(*Starting point*)
	If[aEQ!={},xk=LinearSolve[aEQ,bEQ],xk=Table[1,{n}]];
	If[aLE!={},leCons=aLE.xk-bLE];
	If[Max[leCons]>0,
		xk = phaseILP[aEQ,bEQ,aLE,bLE,prResults];
		If[aLE!={},leCons=aLE.xk-bLE]
	];
	If[Max[leCons]>0, Message[ActiveSetQP::"Infeasible"]; Abort[]];
	Map[If[Abs[leCons[[#]]]\[LessEqual]\[Epsilon],AppendTo[wk,#]]&,Range[Length[aLE]]];
	(* Main loop *)
	While[And[status==="NonOptimum",++iter\[LessEqual]maxIter],
		wk = Sort[wk];
		gk=Q.xk+c;
		A=Join[aEQ,aLE[[wk]]];
		If[prResults,
			StylePrint["**** Iteration "<>
				ToString[iter]<>" ****", "Output", FontWeight->"Bold"];
			PrintLabelledList[{{xk}, {wk}},
			{"Current point", "Active set"},1];
			PrintLabelledList[{A, gk},{"A", "gk"},2];
			If[aLE!={},PrintLabelledList[{{aLE.xk-bLE}},
				{"LE Constraint values"}]];
			If[aEQ!={},PrintLabelledList[{{aEQ.xk-bEQ}},
				{"EQ Constraint values"}]]
		];
		If[Length[A]==0,lhs=Q,
			lhs=BlockMatrix[{{Q,Transpose[A]},
			{A,ZeroMatrix[Length[A]]}}]
		];
		rhs=Join[-gk,Table[0,{Length[A]}]];
		sol=LinearSolve[lhs,rhs];
		dk=sol[[Range[n]]]; vk=sol[[Range[n+1,n+Length[aEQ]]]];
		uk=Drop[sol,n+Length[aEQ]];
		AppendTo[hist,xk];
		If[prResults,
			PrintLabelledList[{lhs,rhs},{"Equations: lhs","rhs"},2];
			PrintLabelledList[{dk,uk,vk},{"Solution: dk","uk","vk"},3]
		];
		(* Step length *)
		If[Norm[dk]>tol,
			{\[Alpha],wk}=stepLength[aLE,bLE,dk,xk,wk,\[Epsilon],prResults];
			xk = xk + \[Alpha] dk;
			If[prResults,
				PrintLabelledList[{{xk}},{"New point"}]
			],
			If[Or[Length[uk]===0, Min[uk]\[GreaterEqual]0],
				status="Optimum",
				wk=Complement[wk,
					wk[[Position[uk,Min[uk]][[1]]]]]
			];
		];
	];
	uAll=Table[0,{Length[bLE]}];
	If[Length[uk]\[GreaterEqual]1,uAll[[wk]] = uk];
	{{xk,uAll,vk},status,hist,If[status==="NonOptimum",iter-1,iter]}
];

stepLength[a_,b_,d_,xk_,wk_,\[Epsilon]_,prResults_]:=
Module[{\[Alpha]=1,ratios={},imin,nwk,in,ad,i,newWk=wk, cons},
	nwk = Complement[Range[Length[a]],wk];
	If[Length[nwk]>0,
		ad = a[[nwk]].d;
		cons = b[[nwk]]-a[[nwk]].xk;
		ratios = Table[If[ad[[i]]>0, cons[[i]]/ad[[i]],\[Infinity]],
			{i,1,Length[ad]}];
		If[prResults,
			PrintLabelledList[{{nwk}},{"Inactive set"}];
			PrintLabelledList[{{cons}},{"bi-Ai.x (i\[Element]Inactive)"}];
			PrintLabelledList[{{ad}},{"Ai.d (i\[Element]Inactive)"}];
			PrintLabelledList[{{ratios}},{"bi-Ai.x/Ai.d"}]
		];
		If[ratios!={},
			\[Alpha] = Min[1,Select[ratios,Positive]];
			If[\[Alpha]<1,imin=Position[ratios,\[Alpha]];
				newWk=Union[Flatten[{wk,imin}]]]
		];
	];
	If[prResults,
		PrintLabelledList[{\[Alpha],{newWk}},
			{"Step length, \[Alpha]", "New active set"}]
	];
	{\[Alpha],newWk}
];	

phaseILP[aEQ_, bEQ_, aLE_, bLE_, prResults_] :=
Module[{A,b,AA,sol,neq=Length[aEQ],nle=Length[aLE],
al=aLE,ae=aEQ,m,n,xk, flp},
(* All constraints are expressed as '\[GreaterEqual]' type since LinearProgramming function expects this form. The number of variables are doubled since LinearProgramming expects all variables to be \[GreaterEqual] 0. Also for ease of implementation all constraints are included in both the objective function and the constraints. *)
	A=Join[aEQ,-aEQ,-aLE];
	{m,n}=Dimensions[A];
	b=Join[bEQ,-bEQ,-bLE];
	AA=AppendRows[A,-A,IdentityMatrix[m]];
	flp = Flatten[{Table[0,{2n}],Table[1,{m}]}];
	sol=LinearProgramming[flp,AA,b];
	If[prResults,
		Print["Phase I problem: Art. Obj. coefficients"-> flp];
		Print["Constraints expressed as GE type"];
		PrintLabelledList[{AA,{b}},
			{"Constraint coefficients","Constraint rhs"},1]
	];	
	If[MemberQ[Map[NumericQ,sol],False],
		(* In case Simplex fails, try Primal Affine *)
		Message[ActiveSetQP::"Phase-I"];
		If[nle!=0,
			al=AppendRows[aLE,-aLE,IdentityMatrix[nle]];
			If[neq!=0,ae=AppendRows[aEQ,-aEQ,ZeroMatrix[neq,nle]]]
		];
		A=Join[ae,al];
		b=Join[bEQ,bLE];
		(*PrintLabelledList[{A,b},{"A","b"}];*)
		sol=PhaseIPrimalAffineLP[A,b];
	];
	If[prResults,
		Print["Phase I-LP solution"->sol]
	];	
	xk=Apply[Subtract,{Take[sol,n],Take[sol,{n+1,2n}]}];
	xk
];

PhaseIPrimalAffineLP[A_,b_]:=
Module[{m,n,xk,status="NonOptimum",iter=0,
maxIter=10,x1,\[Beta]=1.0,tol=0.000001,intResults,A1},
	{m,n}=Dimensions[A];
	xk=Table[1,{n}];
	x1=Join[xk,{1}];
	While[And[status==="NonOptimum",iter\[LessEqual]maxIter],
		++iter;
		A1=Transpose[Join[Transpose[A],{b-A.xk}]];
		c1=Join[Table[0,{n}],{Last[x1]}];
		{x1,status,intResults}=PrimalAffineStep[
				n+1,A1,b,c1,x1,\[Beta],tol];
		xk=Drop[x1,-1]
	];
	If[Not[status==="Optimum"],
		Message[ActiveSetQP::"Infeasible"];
		Abort[]
	];
	xk
]

PrimalAffineStep[n_,A_,b_,c_,x_,\[Beta]_,\[Epsilon]_]:=
Module[{newx=x,Tk=DiagonalMatrix[x],w,ATk,AT=Transpose[A],
	status="NonOptimum",r,\[Alpha]=0,intResults,\[Sigma]p,\[Sigma]d,\[Sigma]c,d},
	ATk=A.(Tk^2);
	w=LinearSolve[ATk.AT,ATk.c];
	r=c-AT.w;
	\[Sigma]p=Norm[A.x-b]/(Norm[b]+1);
	\[Sigma]d=Norm[r]/(Norm[c]+1);
	\[Sigma]c=Abs[c.x - b.w];
	If[And[Max[\[Sigma]p,\[Sigma]d,\[Sigma]c]\[LessEqual]\[Epsilon],Min[x]\[GreaterEqual]0], status="Optimum",
		d=-Tk.r;
		If[Chop[Min[d]]===0,status="Unbounded",
			If[Norm[d]\[LessEqual]\[Epsilon], status="Optimum",
				\[Alpha]=\[Beta] Min[Select[-1/d,Positive]];
				newx=x+\[Alpha] Tk.d
			]
		]
	];
	intResults={{ATk,w,r,d,Norm[d],\[Alpha]},
		{"A.T^2","w","r","d","Norm[d]","\[Alpha]"}};
	{newx,status,intResults}
];



ActiveSetDualQP[f_, vars_, opts___?OptionQ] :=
Module[{method, eps, maxIter, prLevel, prSetup, prResults,
	x,status="NonOptimum",unrestrictedVars,c0, c, Q, sol={}, fstar={},
	restrictedVars,reOrderedVars,nonNeg, strtVec, type, sf},
	{type, strtVec, unrestrictedVars,prLevel,maxIter, eps, method} = 
		{ProblemType, StartingVector, UnrestrictedVariables,
		 PrintLevel,
		MaxIterations,ConvergenceTolerance,
		Method} /. Flatten[{opts,Options[ActiveSetDualQP]}];
	Switch[prLevel,
		0, {prSetup, prResults}={False, False}, 
		1, {prSetup, prResults}={True, False}, 
		_, {prSetup, prResults}={True, True}
	];
	If[type===Max,sf=-f,sf=f];
	unrestrictedVars=Sort[unrestrictedVars];
	restrictedVars = Complement[vars, unrestrictedVars];
	reOrderedVars=Join[restrictedVars,unrestrictedVars];
	If[prSetup,
		PrintLabelledList[{sf,{Thread[restrictedVars\[GreaterEqual]0]}},
			{"Minimize","Subject to"},1];
		If[unrestrictedVars!={}, 
			PrintLabelledList[{{unrestrictedVars}},
			{"Unrestricted variables"}];
			PrintLabelledList[{{Range[Length[vars]],reOrderedVars}},
			{"Order of variables"}]
		];
	];
	nonNeg=Length[restrictedVars];
	{c0,c,Q}=MakeQuadraticForm[sf,reOrderedVars];
	{x, status} = ActiveSetDualQPSolver[c0, c, Q, nonNeg, maxIter, 
		eps, method, strtVec, reOrderedVars, prResults];
	sol = Thread[vars->(vars/.ToRules[reOrderedVars==x])];
	fstar = f/.sol;
	{{fstar, sol}, status}
];  

ActiveSetDualQPSolver[c0_,c_, Q_, nonNeg_, maxIter_, 
eps_, method_, strtVec_, vars_, prResults_] :=
Module[{n=Length[c],iter = 0, alwaysActive, 
	gradf, x, status="NonOptimum", opt},
	alwaysActive = Range[nonNeg+1,n]; 
    If[strtVec==={},x = Table[1.,{n}], x=strtVec];
	While[status==="NonOptimum" && iter < maxIter,
		++iter;
		activeVars=alwaysActive;
		x = Chop[x];
		gradf = Chop[Q.x + c];
		opt = Join[Table[If[Or[x[[i]]>0 && gradf[[i]]===0,
				x[[i]]===0 && gradf[[i]]\[GreaterEqual]0],True, False],{i,1,nonNeg}],
				Map[gradf[[#]]===0&,alwaysActive]];
		If[prResults,
			StylePrint["QP Solution Optimality Check "<>ToString[iter],
			"Output",FontWeight->"Bold"]; 
			PrintLabelledList[{x,gradf,opt},
			{MatrixForm[vars],"\[Del]f","Optimality status"}]
		];
		If [MemberQ[opt,False],status="NonOptimum"; 
			Do[If[Or[x[[i]]>0,And[x[[i]]===0,gradf[[i]]<0]],
				AppendTo[activeVars,i]],{i,1,nonNeg}];
			If[activeVars==={},
				status="Indeterminate";x = \[Infinity],
				x=ActiveSetDualQPStep[nonNeg,x,Q,c,c0,Sort[activeVars],eps,
					method,maxIter,vars, prResults]
			],
			status="Optimum"
		];
	];
	{x,status}
];  

ActiveSetDualQPStep[nonNeg_,x0_,Q0_,c0_,fcnst_, activeSet_, eps_, method_, maxIter_,vars_, prResults_]:=
Module[{n=Length[c0],x=x0, c=c0, Q=Q0, activeVars = activeSet, 
Done=False,iter=0, soln, gradf, passiveVars, newPassiveVar,currentNonNeg},
	passiveVars = Map[{#}&,Sort[Complement[Range[n],activeVars],Greater]];
	If[passiveVars!={},
		x = Fold[Drop,x,passiveVars]; 
		c=Fold[Drop,c,passiveVars];
		Q=Fold[RemoveRowColumn,Q,passiveVars]
	];
	currentNonNeg = nonNeg;
	While [Not[Done] && iter < maxIter,
		++iter;
		soln = Table[0.,{n}];
		gradf = Q.x + c;
		If[prResults,
			StylePrint["**** QP Solution Pass  "<>ToString[iter]<>" ****",
			"Output",FontWeight->"Bold"]; 
			PrintLabelledList[
				{{vars[[activeVars]]},{x}},
				{"Active variables","Current point"},1];
			PrintLabelledList[{c,Q,gradf},{"c","Q","\[Del]f"},3]
		];
		{x,newPassiveVar} = CGStep[currentNonNeg,x, Q, c, fcnst, 
				maxIter,eps, prResults, method];
		soln[[activeVars]]=x;
		If[newPassiveVar==={},
			Done=True,
			If[prResults,
				PrintLabelledList[{vars[[newPassiveVar]]},
				{"New var to be made passive"}]
			];
			x = Drop[x,newPassiveVar];
			activeVars = Sort[Drop[activeVars,newPassiveVar]];
			currentNonNeg = Max[0,currentNonNeg-1];
			If[activeVars==={},
				Done=True, 
				c=Drop[c,newPassiveVar];
				Q=RemoveRowColumn[Q,newPassiveVar]
			];
		];
	];
	soln
];

CGStep[nonNeg_,x0_, Q_, c_, c0_, maxIter_,eps_, prResults_,
method_] :=
Module[{n=Length[c],iter = 0, d, dNorm, gradfNorm,
	beta=0, prevD, gradf=Q.x0+c, prevGradf, status="NonOptimum",
	\[Alpha]1=0,\[Alpha]2=0,\[Alpha]=0,ratios={}, newPassiveVar={}, x = x0,
	nonNegRange = Range[Min[Length[x0],nonNeg]]},

    prevD = Table[0.,{n}];
	prevGradf = gradf;
    dNorm = gradfNorm = Norm[gradf];
    If[gradfNorm<eps,status="Optimum"];
	While [newPassiveVar==={} && status==="NonOptimum" && iter < maxIter,
		++iter;
		If[method==="FletcherReeves",
			beta = gradf.gradf/(prevGradf.prevGradf),
			beta = (gradf-prevGradf).gradf/(prevGradf.prevGradf)
		]; 
		d = -gradf + beta prevD;
		\[Alpha] = 0; 
		If[prResults,
			StylePrint[">> Conjugate Gradient Iteration "<>ToString[iter],
			"Output",FontSlant->"Italic"]; 
			PrintLabelledList[
			{d, gradf, gradfNorm, beta},
			{"d","\[Del]f", "||\[Del]f||","\[Beta]"},2]
		];
     	If[gradfNorm<eps,status="Optimum", 
     		If[Chop[d.Q.d]==0, status="Unbounded",
	     		\[Alpha]1 = -d.gradf/(d.Q.d);
	     		If[\[Alpha]1<0,
	     			Message[ActiveSetDualQP::"CGDir", x];
	     			Abort[]
	     		];
	     		If[prResults,PrintLabelledList[{\[Alpha]1},{"\[Alpha]1"}]];
	     		If[Min[d[[nonNegRange]]]>0,\[Alpha]=\[Alpha]1,
		     		ratios = -x/d;
	     			\[Alpha]2 = Min[Select[ratios[[nonNegRange]],Positive]];
	     			\[Alpha]=Min[\[Alpha]1,\[Alpha]2];
	     			If[\[Alpha]===\[Alpha]2, newPassiveVar=Position[ratios,\[Alpha]2][[1]]];
	     			If[prResults,
						PrintLabelledList[{{ratios},\[Alpha]2},{"-pt/d","\[Alpha]2"},1]
					];
	     		];
	     	];
	    ];
		x = x + \[Alpha] d;
		If[prResults,
			PrintLabelledList[{\[Alpha],status},{"\[Alpha]","CG-Status"},2];
			PrintLabelledList[{{x},c0+c.x + 1/2 x.Q.x},
				{"New point", "Objective function"},1]
		];
		prevGradf = gradf;
		prevD = d;
		gradf=Q.x+c;
		gradfNorm = Norm[gradf];
	];
	{x, newPassiveVar}
];  



End[];

EndPackage[]